@@ -13,11 +13,18 @@ import (
 	"github.com/evanw/esbuild/pkg/cli"
 )
 
-const helpText = `
-Usage:
+var helpText = func(colors logger.Colors) string {
+	return `
+` + colors.Bold + `Usage:` + colors.Default + `
   esbuild [options] [entry points]
 
-Options:
+` + colors.Bold + `Documentation:` + colors.Default + `
+  ` + colors.Underline + `https://esbuild.github.io/` + colors.Default + `
+
+` + colors.Bold + `Repository:` + colors.Default + `
+  ` + colors.Underline + `https://github.com/evanw/esbuild` + colors.Default + `
+
+` + colors.Bold + `Simple options:` + colors.Default + `
   --bundle              Bundle all dependencies into the output files
   --define:K=V          Substitute K with V while parsing
   --external:M          Exclude module M from the bundle
@@ -37,14 +44,14 @@ Options:
   --outdir=...          The output directory (for multiple entry points)
   --outfile=...         The output file (for one entry point)
   --platform=...        Platform target (browser | node, default browser)
-  --serve=...           Start a local HTTP server on this port for outputs
+  --serve=...           Start a local HTTP server on this host:port for outputs
   --sourcemap           Emit a source map
   --splitting           Enable code splitting (currently only for esm)
+  --summary             Print some helpful information at the end of a build
   --target=...          Environment target (e.g. es2017, chrome58, firefox57,
                         safari11, edge16, node10, default esnext)
 
-Advanced options:
-  --avoid-tdz               An optimization for large bundles in Safari
+` + colors.Bold + `Advanced options:` + colors.Default + `
   --banner=...              Text to be prepended to each output file
   --charset=utf8            Do not escape UTF-8 code points
   --color=...               Force use of color terminal escapes (true | false)
@@ -69,24 +76,27 @@ Advanced options:
   --sourcefile=...          Set the source file for the source map (for stdin)
   --sourcemap=external      Do not link to the source map with a comment
   --sourcemap=inline        Emit the source map with an inline data URL
+  --sources-content=false   Omit "sourcesContent" in generated source maps
   --tree-shaking=...        Set to "ignore-annotations" to work with packages
                             that have incorrect tree-shaking annotations
   --tsconfig=...            Use this tsconfig.json file instead of other ones
   --version                 Print the current version and exit (` + esbuildVersion + `)
 
-Examples:
-  # Produces dist/entry_point.js and dist/entry_point.js.map
+` + colors.Bold + `Examples:` + colors.Default + `
+  ` + colors.Dim + `# Produces dist/entry_point.js and dist/entry_point.js.map` + colors.Default + `
   esbuild --bundle entry_point.js --outdir=dist --minify --sourcemap
 
-  # Allow JSX syntax in .js files
+  ` + colors.Dim + `# Allow JSX syntax in .js files` + colors.Default + `
   esbuild --bundle entry_point.js --outfile=out.js --loader:.js=jsx
 
-  # Substitute the identifier RELEASE for the literal true
+  ` + colors.Dim + `# Substitute the identifier RELEASE for the literal true` + colors.Default + `
   esbuild example.js --outfile=out.js --define:RELEASE=true
 
-  # Provide input via stdin, get output via stdout
+  ` + colors.Dim + `# Provide input via stdin, get output via stdout` + colors.Default + `
   esbuild --minify --loader=ts < input.ts > output.js
+
 `
+}
 
 func main() {
 	osArgs := os.Args[1:]
@@ -101,7 +111,7 @@ func main() {
 		switch {
 		// Show help if a common help flag is provided
 		case arg == "-h", arg == "-help", arg == "--help", arg == "/?":
-			fmt.Fprintf(os.Stderr, "%s\n", helpText)
+			logger.PrintText(os.Stdout, logger.LevelSilent, os.Args, helpText)
 			os.Exit(0)
 
 		// Special-case the version flag here
@@ -150,7 +160,7 @@ func main() {
 
 	// Print help text when there are no arguments
 	if len(osArgs) == 0 && logger.GetTerminalInfo(os.Stdin).IsTTY {
-		fmt.Fprintf(os.Stderr, "%s\n", helpText)
+		logger.PrintText(os.Stdout, logger.LevelSilent, osArgs, helpText)
 		os.Exit(0)
 	}
 
