@@ -15,9 +15,9 @@ import (
 	"runtime/debug"
 	"sync"
 
+	"github.com/evanw/esbuild/internal/cli_helpers"
 	"github.com/evanw/esbuild/internal/config"
 	"github.com/evanw/esbuild/internal/fs"
-	"github.com/evanw/esbuild/internal/helpers"
 	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/pkg/api"
 	"github.com/evanw/esbuild/pkg/cli"
@@ -316,12 +316,12 @@ func (service *serviceType) handleBuildRequest(id uint32, request map[string]int
 	}
 
 	// Optionally allow input from the stdin channel
-	if stdin, ok := request["stdin"].(string); ok {
+	if stdin, ok := request["stdinContents"].(string); ok {
 		if options.Stdin == nil {
 			options.Stdin = &api.StdinOptions{}
 		}
 		options.Stdin.Contents = stdin
-		if resolveDir, ok := request["resolveDir"].(string); ok {
+		if resolveDir, ok := request["stdinResolveDir"].(string); ok {
 			options.Stdin.ResolveDir = resolveDir
 		}
 	}
@@ -488,7 +488,7 @@ func (service *serviceType) handleBuildRequest(id uint32, request map[string]int
 						result.Warnings = decodeMessages(value.([]interface{}))
 					}
 					if value, ok := response["loader"]; ok {
-						loader, err := helpers.ParseLoader(value.(string))
+						loader, err := cli_helpers.ParseLoader(value.(string))
 						if err != nil {
 							return api.OnLoadResult{}, err
 						}
@@ -508,6 +508,9 @@ func (service *serviceType) handleBuildRequest(id uint32, request map[string]int
 		if port, ok := serve["port"]; ok {
 			serveOptions.Port = uint16(port.(int))
 		}
+		if host, ok := serve["host"]; ok {
+			serveOptions.Host = host.(string)
+		}
 		serveOptions.OnRequest = func(args api.ServeOnRequestArgs) {
 			service.sendRequest(map[string]interface{}{
 				"command": "serve-request",
@@ -527,6 +530,7 @@ func (service *serviceType) handleBuildRequest(id uint32, request map[string]int
 		}
 		response := map[string]interface{}{
 			"port": int(result.Port),
+			"host": result.Host,
 		}
 
 		// Asynchronously wait for the server to stop, then fulfil the "wait" promise
