@@ -183,9 +183,9 @@ func TestPackageJsonSideEffectsFalseKeepBareImportAndRequireES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `/Users/user/project/src/entry.js: warning: Ignoring this import because "` +
-			`/Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
-/Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
+		expectedScanLog: `Users/user/project/src/entry.js: warning: Ignoring this import because "` +
+			`Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
 			`in the enclosing "package.json" file
 `,
 	})
@@ -214,9 +214,9 @@ func TestPackageJsonSideEffectsFalseKeepBareImportAndRequireCommonJS(t *testing.
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `/Users/user/project/src/entry.js: warning: Ignoring this import because "` +
-			`/Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
-/Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
+		expectedScanLog: `Users/user/project/src/entry.js: warning: Ignoring this import because "` +
+			`Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
 			`in the enclosing "package.json" file
 `,
 	})
@@ -244,9 +244,9 @@ func TestPackageJsonSideEffectsFalseRemoveBareImportES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `/Users/user/project/src/entry.js: warning: Ignoring this import because "` +
-			`/Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
-/Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
+		expectedScanLog: `Users/user/project/src/entry.js: warning: Ignoring this import because "` +
+			`Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
 			`in the enclosing "package.json" file
 `,
 	})
@@ -274,9 +274,9 @@ func TestPackageJsonSideEffectsFalseRemoveBareImportCommonJS(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `/Users/user/project/src/entry.js: warning: Ignoring this import because "` +
-			`/Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
-/Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
+		expectedScanLog: `Users/user/project/src/entry.js: warning: Ignoring this import because "` +
+			`Users/user/project/node_modules/demo-pkg/index.js" was marked as having no side effects
+Users/user/project/node_modules/demo-pkg/package.json: note: "sideEffects" is false ` +
 			`in the enclosing "package.json" file
 `,
 	})
@@ -722,9 +722,9 @@ func TestPackageJsonSideEffectsArrayGlob(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `/Users/user/project/src/entry.js: warning: Ignoring this import because ` +
-			`"/Users/user/project/node_modules/demo-pkg/remove/this/file.js" was marked as having no side effects
-/Users/user/project/node_modules/demo-pkg/package.json: note: It was excluded from the "sideEffects" ` +
+		expectedScanLog: `Users/user/project/src/entry.js: warning: Ignoring this import because ` +
+			`"Users/user/project/node_modules/demo-pkg/remove/this/file.js" was marked as having no side effects
+Users/user/project/node_modules/demo-pkg/package.json: note: It was excluded from the "sideEffects" ` +
 			`array in the enclosing "package.json" file
 `,
 	})
@@ -983,8 +983,8 @@ func TestTreeShakingReactElements(t *testing.T) {
 
 func TestDisableTreeShaking(t *testing.T) {
 	defines := config.ProcessDefines(map[string]config.DefineData{
-		"pure":    config.DefineData{CallCanBeUnwrappedIfUnused: true},
-		"some.fn": config.DefineData{CallCanBeUnwrappedIfUnused: true},
+		"pure":    {CallCanBeUnwrappedIfUnused: true},
+		"some.fn": {CallCanBeUnwrappedIfUnused: true},
 	})
 	dce_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1021,3 +1021,193 @@ func TestDisableTreeShaking(t *testing.T) {
 		},
 	})
 }
+
+func TestDeadCodeFollowingJump(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				function testReturn() {
+					if (true) return y + z()
+					if (FAIL) return FAIL
+					if (x) { var y }
+					function z() { KEEP_ME() }
+					return FAIL
+				}
+
+				function testThrow() {
+					if (true) throw y + z()
+					if (FAIL) return FAIL
+					if (x) { var y }
+					function z() { KEEP_ME() }
+					return FAIL
+				}
+
+				function testBreak() {
+					while (true) {
+						if (true) {
+							y + z()
+							break
+						}
+						if (FAIL) return FAIL
+						if (x) { var y }
+						function z() { KEEP_ME() }
+						return FAIL
+					}
+				}
+
+				function testContinue() {
+					while (true) {
+						if (true) {
+							y + z()
+							continue
+						}
+						if (FAIL) return FAIL
+						if (x) { var y }
+						function z() { KEEP_ME() }
+						return FAIL
+					}
+				}
+
+				function testStmts() {
+					return [a, b, c, d, e, f, g, h, i]
+
+					while (x) { var a }
+					while (FAIL) { let FAIL }
+
+					do { var b } while (x)
+					do { let FAIL } while (FAIL)
+
+					for (var c; ;) ;
+					for (let FAIL; ;) ;
+
+					for (var d in x) ;
+					for (let FAIL in FAIL) ;
+
+					for (var e of x) ;
+					for (let FAIL of FAIL) ;
+
+					if (x) { var f }
+					if (FAIL) { let FAIL }
+
+					if (x) ; else { var g }
+					if (FAIL) ; else { let FAIL }
+
+					{ var h }
+					{ let FAIL }
+
+					x: { var i }
+					x: { let FAIL }
+				}
+
+				testReturn()
+				testThrow()
+				testBreak()
+				testContinue()
+				testStmts()
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MangleSyntax:  true,
+		},
+	})
+}
+
+func TestRemoveTrailingReturn(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				function foo() {
+					if (a) b()
+					return
+				}
+				function bar() {
+					if (a) b()
+					return KEEP_ME
+				}
+				export default [
+					foo,
+					bar,
+					function () {
+						if (a) b()
+						return
+					},
+					function () {
+						if (a) b()
+						return KEEP_ME
+					},
+					() => {
+						if (a) b()
+						return
+					},
+					() => {
+						if (a) b()
+						return KEEP_ME
+					},
+				]
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MangleSyntax:  true,
+			OutputFormat:  config.FormatESModule,
+		},
+	})
+}
+
+func TestImportReExportOfNamespaceImport(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/entry.js": `
+				import * as ns from 'pkg'
+				console.log(ns.foo)
+			`,
+			"/Users/user/project/node_modules/pkg/index.js": `
+				export { default as foo } from './foo'
+				export { default as bar } from './bar'
+			`,
+			"/Users/user/project/node_modules/pkg/package.json": `
+				{ "sideEffects": false }
+			`,
+			"/Users/user/project/node_modules/pkg/foo.js": `
+				module.exports = 123
+			`,
+			"/Users/user/project/node_modules/pkg/bar.js": `
+				module.exports = 'abc'
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestTreeShakingImportIdentifier(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import * as a from './a'
+				new a.Keep()
+			`,
+			"/a.js": `
+				import * as b from './b'
+				export class Keep extends b.Base {}
+				export class REMOVE extends b.Base {}
+			`,
+			"/b.js": `
+				export class Base {}
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
