@@ -512,7 +512,63 @@ func TestImportExportStarAmbiguousError(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: "/entry.js: error: Ambiguous import \"y\" has multiple matching exports\n",
+		expectedCompileLog: `entry.js: error: Ambiguous import "y" has multiple matching exports
+`,
+	})
+}
+
+func TestReExportStarNameCollisionNotAmbiguousImport(t *testing.T) {
+	importstar_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import {x, y} from './common'
+				console.log(x, y)
+			`,
+			"/common.js": `
+				export * from './a'
+				export * from './b'
+			`,
+			"/a.js": `
+				export * from './c'
+			`,
+			"/b.js": `
+				export {x} from './c'
+			`,
+			"/c.js": `
+				export let x = 1, y = 2
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
+func TestReExportStarNameCollisionNotAmbiguousExport(t *testing.T) {
+	importstar_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				export * from './a'
+				export * from './b'
+			`,
+			"/a.js": `
+				export * from './c'
+			`,
+			"/b.js": `
+				export {x} from './c'
+			`,
+			"/c.js": `
+				export let x = 1, y = 2
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			OutputFormat:  config.FormatESModule,
+			AbsOutputFile: "/out.js",
+		},
 	})
 }
 
@@ -612,6 +668,24 @@ func TestExportSelfIIFE(t *testing.T) {
 	})
 }
 
+func TestExportSelfIIFEWithName(t *testing.T) {
+	importstar_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				export const foo = 123
+				export * from './entry'
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			OutputFormat:  config.FormatIIFE,
+			AbsOutputFile: "/out.js",
+			GlobalName:    []string{"someName"},
+		},
+	})
+}
+
 func TestExportSelfES6(t *testing.T) {
 	importstar_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -906,7 +980,7 @@ func TestNamespaceImportMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `/entry.js: warning: No matching export for import "foo"
+		expectedCompileLog: `entry.js: warning: No matching export for import "foo"
 `,
 	})
 }
@@ -968,7 +1042,7 @@ func TestNamespaceImportUnusedMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `/entry.js: warning: No matching export for import "foo"
+		expectedCompileLog: `entry.js: warning: No matching export for import "foo"
 `,
 	})
 }
@@ -1074,8 +1148,8 @@ func TestNamespaceImportReExportMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `/foo.js: error: No matching export for import "foo"
-/foo.js: error: No matching export for import "foo"
+		expectedCompileLog: `foo.js: error: No matching export for import "foo"
+foo.js: error: No matching export for import "foo"
 `,
 	})
 }
@@ -1099,8 +1173,8 @@ func TestNamespaceImportReExportUnusedMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `/foo.js: error: No matching export for import "foo"
-/foo.js: error: No matching export for import "foo"
+		expectedCompileLog: `foo.js: error: No matching export for import "foo"
+foo.js: error: No matching export for import "foo"
 `,
 	})
 }
@@ -1124,7 +1198,7 @@ func TestNamespaceImportReExportStarMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `/entry.js: warning: No matching export for import "foo"
+		expectedCompileLog: `entry.js: warning: No matching export for import "foo"
 `,
 	})
 }
@@ -1148,7 +1222,7 @@ func TestNamespaceImportReExportStarUnusedMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `/entry.js: warning: No matching export for import "foo"
+		expectedCompileLog: `entry.js: warning: No matching export for import "foo"
 `,
 	})
 }
@@ -1210,7 +1284,7 @@ func TestReExportStarExternalIIFE(t *testing.T) {
 			Mode:          config.ModeBundle,
 			OutputFormat:  config.FormatIIFE,
 			AbsOutputFile: "/out.js",
-			ModuleName:    []string{"mod"},
+			GlobalName:    []string{"mod"},
 			ExternalModules: config.ExternalModules{
 				NodeModules: map[string]bool{
 					"foo": true,
@@ -1274,7 +1348,7 @@ func TestReExportStarIIFENoBundle(t *testing.T) {
 			Mode:          config.ModeConvertFormat,
 			OutputFormat:  config.FormatIIFE,
 			AbsOutputFile: "/out.js",
-			ModuleName:    []string{"mod"},
+			GlobalName:    []string{"mod"},
 		},
 	})
 }
@@ -1323,7 +1397,7 @@ func TestReExportStarAsExternalIIFE(t *testing.T) {
 			Mode:          config.ModeBundle,
 			OutputFormat:  config.FormatIIFE,
 			AbsOutputFile: "/out.js",
-			ModuleName:    []string{"mod"},
+			GlobalName:    []string{"mod"},
 			ExternalModules: config.ExternalModules{
 				NodeModules: map[string]bool{
 					"foo": true,
@@ -1387,7 +1461,7 @@ func TestReExportStarAsIIFENoBundle(t *testing.T) {
 			Mode:          config.ModeConvertFormat,
 			OutputFormat:  config.FormatIIFE,
 			AbsOutputFile: "/out.js",
-			ModuleName:    []string{"mod"},
+			GlobalName:    []string{"mod"},
 		},
 	})
 }
@@ -1502,8 +1576,8 @@ func TestImportDefaultNamespaceComboIssue446(t *testing.T) {
 				},
 			},
 		},
-		expectedCompileLog: `/internal-def.js: warning: No matching export for import "def"
-/internal-ns-def.js: warning: No matching export for import "def"
+		expectedCompileLog: `internal-def.js: warning: No matching export for import "def"
+internal-ns-def.js: warning: No matching export for import "def"
 `,
 	})
 }
@@ -1534,13 +1608,13 @@ func TestImportDefaultNamespaceComboNoDefault(t *testing.T) {
 				},
 			},
 		},
-		expectedCompileLog: `/entry-default-ns-prop.js: error: No matching export for import "default"
-/entry-default-ns-prop.js: warning: No matching export for import "default"
-/entry-default-ns.js: error: No matching export for import "default"
-/entry-default-prop.js: error: No matching export for import "default"
-/entry-default-prop.js: warning: No matching export for import "default"
-/entry-default.js: error: No matching export for import "default"
-/entry-prop.js: warning: No matching export for import "default"
+		expectedCompileLog: `entry-default-ns-prop.js: error: No matching export for import "default"
+entry-default-ns-prop.js: warning: No matching export for import "default"
+entry-default-ns.js: error: No matching export for import "default"
+entry-default-prop.js: error: No matching export for import "default"
+entry-default-prop.js: warning: No matching export for import "default"
+entry-default.js: error: No matching export for import "default"
+entry-prop.js: warning: No matching export for import "default"
 `,
 	})
 }
