@@ -2,7 +2,6 @@ package js_ast
 
 import (
 	"sort"
-	"strings"
 
 	"github.com/evanw/esbuild/internal/ast"
 	"github.com/evanw/esbuild/internal/compat"
@@ -379,9 +378,14 @@ type EUnary struct {
 }
 
 type EBinary struct {
-	Op    OpCode
 	Left  Expr
 	Right Expr
+	Op    OpCode
+
+	// If true, this expression was originally a self-assignment of the form
+	// "x = y". This is relevant because it's used to emit a warning later on
+	// about code that accidentally does "x = x".
+	WasIdentifierAssign bool
 }
 
 type EBoolean struct{ Value bool }
@@ -503,6 +507,10 @@ type EIdentifier struct {
 	// unwrapped if the resulting value is unused. Unwrapping means discarding
 	// the call target but keeping any arguments with side effects.
 	CallCanBeUnwrappedIfUnused bool
+
+	// If true, this identifier was originally a "x as T" or "<T>x" expression.
+	// This is used to avoid warnings about "x = x as T" assignments.
+	WasTypeScriptCast bool
 }
 
 // This is similar to an EIdentifier but it represents a reference to an ES6
@@ -649,11 +657,11 @@ func (*ERequireResolve) isExpr()    {}
 func (*EImport) isExpr()            {}
 
 func Assign(a Expr, b Expr) Expr {
-	return Expr{a.Loc, &EBinary{BinOpAssign, a, b}}
+	return Expr{Loc: a.Loc, Data: &EBinary{Op: BinOpAssign, Left: a, Right: b}}
 }
 
 func AssignStmt(a Expr, b Expr) Stmt {
-	return Stmt{a.Loc, &SExpr{Value: Assign(a, b)}}
+	return Stmt{Loc: a.Loc, Data: &SExpr{Value: Assign(a, b)}}
 }
 
 func Not(a Expr) Expr {
@@ -661,7 +669,7 @@ func Not(a Expr) Expr {
 	if not, ok := a.Data.(*EUnary); ok && not.Op == UnOpNot && IsBooleanValue(not.Value) {
 		return not.Value
 	}
-	return Expr{a.Loc, &EUnary{UnOpNot, a}}
+	return Expr{Loc: a.Loc, Data: &EUnary{UnOpNot, a}}
 }
 
 func IsBooleanValue(a Expr) bool {
@@ -686,7 +694,7 @@ func IsBooleanValue(a Expr) bool {
 }
 
 func JoinWithComma(a Expr, b Expr) Expr {
-	return Expr{a.Loc, &EBinary{BinOpComma, a, b}}
+	return Expr{Loc: a.Loc, Data: &EBinary{Op: BinOpComma, Left: a, Right: b}}
 }
 
 func JoinAllWithComma(all []Expr) Expr {
@@ -1374,7 +1382,7 @@ type AST struct {
 	// since we already have to traverse the AST then anyway and the parser pass
 	// is conveniently fully parallelized.
 	NamedImports            map[Ref]NamedImport
-	NamedExports            map[string]Ref
+	NamedExports            map[string]NamedExport
 	TopLevelSymbolToParts   map[Ref][]uint32
 	ExportStarImportRecords []uint32
 
@@ -1505,6 +1513,11 @@ type NamedImport struct {
 	IsExported bool
 }
 
+type NamedExport struct {
+	Ref      Ref
+	AliasLoc logger.Loc
+}
+
 // Each file is made up of multiple parts, and each part consists of one or
 // more top-level statements. Parts are used for tree shaking and code
 // splitting analysis. Individual parts of a file can be discarded by tree
@@ -1536,7 +1549,7 @@ type Part struct {
 	CanBeRemovedIfUnused bool
 
 	// If true, this is the automatically-generated part for this file's ES6
-	// exports. It may hold the "const exports = {};" statement and also the
+	// exports. It may hold the "var exports = {};" statement and also the
 	// "__export(exports, { ... })" call to initialize the getters.
 	IsNamespaceExport bool
 
@@ -1615,37 +1628,6 @@ func MergeSymbols(symbols SymbolMap, old Ref, new Ref) Ref {
 	return new
 }
 
-// This has a custom implementation instead of using "filepath.Dir/Base/Ext"
-// because it should work the same on Unix and Windows. These names end up in
-// the generated output and the generated output should not depend on the OS.
-func PlatformIndependentPathDirBaseExt(path string) (dir string, base string, ext string) {
-	for {
-		i := strings.LastIndexAny(path, "/\\")
-
-		// Stop if there are no more slashes
-		if i < 0 {
-			base = path
-			break
-		}
-
-		// Stop if we found a non-trailing slash
-		if i+1 != len(path) {
-			dir, base = path[:i], path[i+1:]
-			break
-		}
-
-		// Ignore trailing slashes
-		path = path[:i]
-	}
-
-	// Strip off the extension
-	if dot := strings.LastIndexByte(base, '.'); dot >= 0 {
-		base, ext = base[:dot], base[dot:]
-	}
-
-	return
-}
-
 // For readability, the names of certain automatically-generated symbols are
 // derived from the file name. For example, instead of the CommonJS wrapper for
 // a file being called something like "require273" it can be called something
@@ -1659,19 +1641,23 @@ func PlatformIndependentPathDirBaseExt(path string) (dir string, base string, ex
 // collisions.
 func GenerateNonUniqueNameFromPath(path string) string {
 	// Get the file name without the extension
-	dir, base, _ := PlatformIndependentPathDirBaseExt(path)
+	dir, base, _ := logger.PlatformIndependentPathDirBaseExt(path)
 
 	// If the name is "index", use the directory name instead. This is because
 	// many packages in npm use the file name "index.js" because it triggers
 	// node's implicit module resolution rules that allows you to import it by
 	// just naming the directory.
 	if base == "index" {
-		_, dirBase, _ := PlatformIndependentPathDirBaseExt(dir)
+		_, dirBase, _ := logger.PlatformIndependentPathDirBaseExt(dir)
 		if dirBase != "" {
 			base = dirBase
 		}
 	}
 
+	return EnsureValidIdentifier(base)
+}
+
+func EnsureValidIdentifier(base string) string {
 	// Convert it to an ASCII identifier. Note: If you change this to a non-ASCII
 	// identifier, you're going to potentially cause trouble with non-BMP code
 	// points in target environments that don't support bracketed Unicode escapes.
