@@ -90,7 +90,7 @@ const (
 	TSlash
 	TTilde
 
-	// Assignments
+	// Assignments (keep in sync with IsAssign() below)
 	TAmpersandAmpersandEquals
 	TAmpersandEquals
 	TAsteriskAsteriskEquals
@@ -154,6 +154,10 @@ const (
 	TWith
 )
 
+func (t T) IsAssign() bool {
+	return t >= TAmpersandAmpersandEquals && t <= TSlashEquals
+}
+
 var Keywords = map[string]T{
 	// Reserved words
 	"break":      TBreak,
@@ -223,6 +227,7 @@ type Lexer struct {
 	HasPureCommentBefore            bool
 	PreserveAllCommentsBefore       bool
 	CommentsToPreserveBefore        []js_ast.Comment
+	AllOriginalComments             []js_ast.Comment
 	codePoint                       rune
 	StringLiteral                   []uint16
 	Identifier                      string
@@ -633,9 +638,17 @@ func RangeOfIdentifier(source logger.Source, loc logger.Loc) logger.Range {
 		return logger.Range{Loc: loc, Len: 0}
 	}
 
-	if c, _ := utf8.DecodeRuneInString(text); IsIdentifierStart(c) || c == '\\' {
+	i := 0
+	c, _ := utf8.DecodeRuneInString(text[i:])
+
+	// Handle private names
+	if c == '#' {
+		i++
+		c, _ = utf8.DecodeRuneInString(text[i:])
+	}
+
+	if IsIdentifierStart(c) || c == '\\' {
 		// Search for the end of the identifier
-		i := 0
 		for i < len(text) {
 			c2, width2 := utf8.DecodeRuneInString(text[i:])
 			if c2 == '\\' {
@@ -817,6 +830,7 @@ func (lexer *Lexer) NextInsideJSXElement() {
 
 			case '*':
 				lexer.step()
+				startRange := lexer.Range()
 			multiLineComment:
 				for {
 					switch lexer.codePoint {
@@ -833,7 +847,8 @@ func (lexer *Lexer) NextInsideJSXElement() {
 
 					case -1: // This indicates the end of the file
 						lexer.start = lexer.end
-						lexer.addError(lexer.Loc(), "Expected \"*/\" to terminate multi-line comment")
+						lexer.addErrorWithNotes(lexer.Loc(), "Expected \"*/\" to terminate multi-line comment",
+							[]logger.MsgData{logger.RangeData(&lexer.source, startRange, "The multi-line comment starts here")})
 						panic(LexerPanic{})
 
 					default:
@@ -1215,6 +1230,7 @@ func (lexer *Lexer) Next() {
 
 			case '*':
 				lexer.step()
+				startRange := lexer.Range()
 			multiLineComment:
 				for {
 					switch lexer.codePoint {
@@ -1231,7 +1247,8 @@ func (lexer *Lexer) Next() {
 
 					case -1: // This indicates the end of the file
 						lexer.start = lexer.end
-						lexer.addError(lexer.Loc(), "Expected \"*/\" to terminate multi-line comment")
+						lexer.addErrorWithNotes(lexer.Loc(), "Expected \"*/\" to terminate multi-line comment",
+							[]logger.MsgData{logger.RangeData(&lexer.source, startRange, "The multi-line comment starts here")})
 						panic(LexerPanic{})
 
 					default:
@@ -2318,6 +2335,12 @@ func (lexer *Lexer) addError(loc logger.Loc, text string) {
 	}
 }
 
+func (lexer *Lexer) addErrorWithNotes(loc logger.Loc, text string, notes []logger.MsgData) {
+	if !lexer.IsLogDisabled {
+		lexer.log.AddErrorWithNotes(&lexer.source, loc, text, notes)
+	}
+}
+
 func (lexer *Lexer) addRangeError(r logger.Range, text string) {
 	if !lexer.IsLogDisabled {
 		lexer.log.AddRangeError(&lexer.source, r, text)
@@ -2396,6 +2419,13 @@ func (lexer *Lexer) scanCommentText() {
 	text := lexer.source.Contents[lexer.start:lexer.end]
 	hasPreserveAnnotation := len(text) > 2 && text[2] == '!'
 
+	// Save the original comment text so we can subtract comments from the
+	// character frequency analysis used by symbol minification
+	lexer.AllOriginalComments = append(lexer.AllOriginalComments, js_ast.Comment{
+		Loc:  logger.Loc{Start: int32(lexer.start)},
+		Text: text,
+	})
+
 	for i, n := 0, len(text); i < n; i++ {
 		switch text[i] {
 		case '#':
