@@ -154,6 +154,16 @@ func (p *parser) lowerFunction(
 	hasRestArg *bool,
 	isArrow bool,
 ) {
+	// Size optimization: trim an implicit return from the end of the function body
+	if p.options.mangleSyntax {
+		if n := len(*bodyStmts); n > 0 {
+			last := (*bodyStmts)[n-1]
+			if s, ok := last.Data.(*js_ast.SReturn); ok && s.Value == nil {
+				*bodyStmts = (*bodyStmts)[:n-1]
+			}
+		}
+	}
+
 	// Lower object rest binding patterns in function arguments
 	if p.options.unsupportedJSFeatures.Has(compat.ObjectRestSpread) {
 		var prefixStmts []js_ast.Stmt
@@ -347,15 +357,15 @@ func (p *parser) lowerFunction(
 			argRef := p.newSymbol(js_ast.SymbolOther, "key")
 			p.currentScope.Generated = append(p.currentScope.Generated, *p.fnOrArrowDataVisit.superIndexRef, argRef)
 			superIndexStmt := js_ast.Stmt{Loc: bodyLoc, Data: &js_ast.SLocal{
-				Decls: []js_ast.Decl{js_ast.Decl{
+				Decls: []js_ast.Decl{{
 					Binding: js_ast.Binding{Loc: bodyLoc, Data: &js_ast.BIdentifier{Ref: *p.fnOrArrowDataVisit.superIndexRef}},
 					Value: &js_ast.Expr{Loc: bodyLoc, Data: &js_ast.EArrow{
-						Args: []js_ast.Arg{js_ast.Arg{
+						Args: []js_ast.Arg{{
 							Binding: js_ast.Binding{Loc: bodyLoc, Data: &js_ast.BIdentifier{Ref: argRef}},
 						}},
 						Body: js_ast.FnBody{
 							Loc: bodyLoc,
-							Stmts: []js_ast.Stmt{js_ast.Stmt{Loc: bodyLoc, Data: &js_ast.SReturn{
+							Stmts: []js_ast.Stmt{{Loc: bodyLoc, Data: &js_ast.SReturn{
 								Value: &js_ast.Expr{Loc: bodyLoc, Data: &js_ast.EIndex{
 									Target: js_ast.Expr{Loc: bodyLoc, Data: &js_ast.ESuper{}},
 									Index:  js_ast.Expr{Loc: bodyLoc, Data: &js_ast.EIdentifier{Ref: argRef}},
@@ -374,7 +384,7 @@ func (p *parser) lowerFunction(
 	}
 }
 
-func (p *parser) lowerOptionalChain(expr js_ast.Expr, in exprIn, out exprOut, thisArgFunc func() js_ast.Expr) (js_ast.Expr, exprOut) {
+func (p *parser) lowerOptionalChain(expr js_ast.Expr, in exprIn, childOut exprOut) (js_ast.Expr, exprOut) {
 	valueWhenUndefined := js_ast.Expr{Loc: expr.Loc, Data: &js_ast.EUndefined{}}
 	endsWithPropertyAccess := false
 	containsPrivateName := false
@@ -458,12 +468,12 @@ flatten:
 	var thisArg js_ast.Expr
 	var targetWrapFunc func(js_ast.Expr) js_ast.Expr
 	if startsWithCall {
-		if thisArgFunc != nil {
+		if childOut.thisArgFunc != nil {
 			// The initial value is a nested optional chain that ended in a property
 			// access. The nested chain was processed first and has saved the
 			// appropriate value for "this". The callback here will return a
 			// reference to that saved location.
-			thisArg = thisArgFunc()
+			thisArg = childOut.thisArgFunc()
 		} else {
 			// The initial value is a normal expression. If it's a property access,
 			// strip the property off and save the target of the property access to
@@ -541,12 +551,15 @@ flatten:
 	// Step 4: Wrap the starting value by each expression in the chain. We
 	// traverse the chain in reverse because we want to go from the inside out
 	// and the chain was built from the outside in.
+	var parentThisArgFunc func() js_ast.Expr
+	var parentThisArgWrapFunc func(js_ast.Expr) js_ast.Expr
 	var privateThisFunc func() js_ast.Expr
 	var privateThisWrapFunc func(js_ast.Expr) js_ast.Expr
 	for i := len(chain) - 1; i >= 0; i-- {
 		// Save a reference to the value of "this" for our parent ECall
-		if i == 0 && in.storeThisArgForParentOptionalChain != nil && endsWithPropertyAccess {
-			result = in.storeThisArgForParentOptionalChain(result)
+		if i == 0 && in.storeThisArgForParentOptionalChain && endsWithPropertyAccess {
+			parentThisArgFunc, parentThisArgWrapFunc = p.captureValueWithPossibleSideEffects(result.Loc, 2, result)
+			result = parentThisArgFunc()
 		}
 
 		switch e := chain[i].Data.(type) {
@@ -652,7 +665,13 @@ flatten:
 	if targetWrapFunc != nil {
 		result = targetWrapFunc(result)
 	}
-	return result, exprOut{}
+	if childOut.thisArgWrapFunc != nil {
+		result = childOut.thisArgWrapFunc(result)
+	}
+	return result, exprOut{
+		thisArgFunc:     parentThisArgFunc,
+		thisArgWrapFunc: parentThisArgWrapFunc,
+	}
 }
 
 func (p *parser) lowerAssignmentOperator(value js_ast.Expr, callback func(js_ast.Expr, js_ast.Expr) js_ast.Expr) js_ast.Expr {
@@ -1569,6 +1588,9 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr, shadowRef js_ast
 		}
 	}
 
+	// Safari workaround: Automatically avoid TDZ issues when bundling
+	avoidTDZ := p.options.mode == config.ModeBundle && p.currentScope.Parent == nil
+
 	for _, prop := range class.Properties {
 		// Merge parameter decorators with method decorators
 		if p.options.ts.Parse && prop.IsMethod {
@@ -1598,8 +1620,15 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr, shadowRef js_ast
 			!p.options.useDefineForClassFields && !mustLowerPrivate
 
 		// Class fields must be lowered if the environment doesn't support them
-		mustLowerField := !prop.IsMethod && (!prop.IsStatic && p.options.unsupportedJSFeatures.Has(compat.ClassField) ||
-			(prop.IsStatic && p.options.unsupportedJSFeatures.Has(compat.ClassStaticField)))
+		mustLowerField := !prop.IsMethod &&
+			(!prop.IsStatic && p.options.unsupportedJSFeatures.Has(compat.ClassField) ||
+				(prop.IsStatic && p.options.unsupportedJSFeatures.Has(compat.ClassStaticField)))
+
+		// Be conservative and always lower static fields when we're doing TDZ-
+		// avoidance and the shadowing name for the class was captured somewhere.
+		if !prop.IsMethod && prop.IsStatic && avoidTDZ && shadowRef != js_ast.InvalidRef {
+			mustLowerField = true
+		}
 
 		// Make sure the order of computed property keys doesn't change. These
 		// expressions have side effects and must be evaluated in order.
@@ -1981,9 +2010,6 @@ func (p *parser) lowerClass(stmt js_ast.Stmt, expr js_ast.Expr, shadowRef js_ast
 		keepNameStmt = p.keepStmtSymbolName(name.Loc, name.Data.(*js_ast.EIdentifier).Ref, nameToKeep)
 	}
 
-	// Safari workaround: Automatically avoid TDZ issues when bundling
-	avoidTDZ := p.options.mode == config.ModeBundle && p.currentScope.Parent == nil
-
 	// Pack the class back into a statement, with potentially some extra
 	// statements afterwards
 	var stmts []js_ast.Stmt
