@@ -255,6 +255,9 @@ func (p *parser) skipTypeScriptTypePrefix() {
 				p.lexer.Next()
 			}
 			p.skipTypeScriptType(js_ast.LLowest)
+			if p.lexer.Token == js_lexer.TQuestion {
+				p.lexer.Next()
+			}
 			if p.lexer.Token == js_lexer.TColon {
 				p.lexer.Next()
 				p.skipTypeScriptType(js_ast.LLowest)
@@ -303,6 +306,17 @@ func (p *parser) skipTypeScriptTypeSuffix(level js_ast.L) {
 			p.lexer.Next()
 			p.skipTypeScriptType(js_ast.LBitwiseAnd)
 
+		case js_lexer.TExclamation:
+			// A postfix "!" is allowed in JSDoc types in TypeScript, which are only
+			// present in comments. While it's not valid in a non-comment position,
+			// it's still parsed and turned into a soft error by the TypeScript
+			// compiler. It turns out parsing this is important for correctness for
+			// "as" casts because the "!" token must still be consumed.
+			if p.lexer.HasNewlineBefore {
+				return
+			}
+			p.lexer.Next()
+
 		case js_lexer.TDot:
 			p.lexer.Next()
 			if !p.lexer.IsIdentifierOrKeyword() {
@@ -339,28 +353,14 @@ func (p *parser) skipTypeScriptTypeSuffix(level js_ast.L) {
 
 		case js_lexer.TExtends:
 			// "{ x: number \n extends: boolean }" must not become a single type
-			if p.lexer.HasNewlineBefore {
-				return
-			}
-			p.lexer.Next()
-			p.skipTypeScriptType(js_ast.LCompare)
-
-		case js_lexer.TQuestion:
-			if level >= js_ast.LConditional {
+			if p.lexer.HasNewlineBefore || level >= js_ast.LConditional {
 				return
 			}
 			p.lexer.Next()
 
-			switch p.lexer.Token {
-			// Stop now if we're parsing one of these:
-			// "(a?: b) => void"
-			// "(a?, b?) => void"
-			// "(a?) => void"
-			// "[string?]"
-			case js_lexer.TColon, js_lexer.TComma, js_lexer.TCloseParen, js_lexer.TCloseBracket:
-				return
-			}
-
+			// The type following "extends" is not permitted to be another conditional type
+			p.skipTypeScriptType(js_ast.LConditional)
+			p.lexer.Expect(js_lexer.TQuestion)
 			p.skipTypeScriptType(js_ast.LLowest)
 			p.lexer.Expect(js_lexer.TColon)
 			p.skipTypeScriptType(js_ast.LLowest)
@@ -739,7 +739,7 @@ func (p *parser) skipTypeScriptTypeStmt(opts parseStmtOpts) {
 
 func (p *parser) parseTypeScriptDecorators() []js_ast.Expr {
 	var tsDecorators []js_ast.Expr
-	if p.TS.Parse {
+	if p.options.ts.Parse {
 		for p.lexer.Token == js_lexer.TAt {
 			p.lexer.Next()
 
@@ -826,7 +826,7 @@ func (p *parser) parseTypeScriptEnumStmt(loc logger.Loc, opts parseStmtOpts) js_
 func (p *parser) parseTypeScriptImportEqualsStmt(loc logger.Loc, opts parseStmtOpts, defaultNameLoc logger.Loc, defaultName string) js_ast.Stmt {
 	p.lexer.Expect(js_lexer.TEquals)
 
-	kind := js_ast.LocalVar
+	kind := p.selectLocalKind(js_ast.LocalConst)
 	name := p.lexer.Identifier
 	value := js_ast.Expr{Loc: p.lexer.Loc(), Data: &js_ast.EIdentifier{Ref: p.storeNameInRef(name)}}
 	p.lexer.Expect(js_lexer.TIdentifier)
@@ -837,7 +837,6 @@ func (p *parser) parseTypeScriptImportEqualsStmt(loc logger.Loc, opts parseStmtO
 		path := js_ast.Expr{Loc: p.lexer.Loc(), Data: &js_ast.EString{Value: p.lexer.StringLiteral}}
 		p.lexer.Expect(js_lexer.TStringLiteral)
 		p.lexer.Expect(js_lexer.TCloseParen)
-		kind = js_ast.LocalConst
 		value.Data = &js_ast.ECall{
 			Target: value,
 			Args:   []js_ast.Expr{path},
@@ -857,7 +856,7 @@ func (p *parser) parseTypeScriptImportEqualsStmt(loc logger.Loc, opts parseStmtO
 	}
 
 	p.lexer.ExpectOrInsertSemicolon()
-	ref := p.declareSymbol(js_ast.SymbolOther, defaultNameLoc, defaultName)
+	ref := p.declareSymbol(js_ast.SymbolConst, defaultNameLoc, defaultName)
 	decls := []js_ast.Decl{{
 		Binding: js_ast.Binding{Loc: defaultNameLoc, Data: &js_ast.BIdentifier{Ref: ref}},
 		Value:   &value,
