@@ -510,7 +510,7 @@ type printer struct {
 	symbols            js_ast.SymbolMap
 	renamer            renamer.Renamer
 	importRecords      []ast.ImportRecord
-	options            PrintOptions
+	options            Options
 	extractedComments  map[string]bool
 	needsSemicolon     bool
 	js                 []byte
@@ -530,7 +530,7 @@ type printer struct {
 	lastGeneratedUpdate int
 	generatedColumn     int
 	hasPrevState        bool
-	lineOffsetTables    []lineOffsetTable
+	lineOffsetTables    []LineOffsetTable
 
 	// For splicing adjacent files together
 	finalLocalSemi           int
@@ -550,7 +550,7 @@ type printer struct {
 	coverLinesWithoutMappings bool
 }
 
-type lineOffsetTable struct {
+type LineOffsetTable struct {
 	byteOffsetToStartOfLine int32
 
 	// The source map specification is very loose and does not specify what
@@ -586,7 +586,7 @@ func (p *printer) printQuotedUTF8(text string, allowBacktick bool) {
 }
 
 func (p *printer) addSourceMapping(loc logger.Loc) {
-	if p.options.SourceForSourceMap == nil || loc == p.prevLoc {
+	if !p.options.AddSourceMappings || loc == p.prevLoc {
 		return
 	}
 	p.prevLoc = loc
@@ -686,7 +686,7 @@ func (p *printer) updateGeneratedLineAndColumn() {
 	p.lastGeneratedUpdate = len(p.js)
 }
 
-func generateLineOffsetTables(contents string, approximateLineCount int32) []lineOffsetTable {
+func GenerateLineOffsetTables(contents string, approximateLineCount int32) []LineOffsetTable {
 	var columnsForNonASCII []int32
 	byteOffsetToFirstNonASCII := int32(0)
 	lineByteOffset := 0
@@ -694,7 +694,7 @@ func generateLineOffsetTables(contents string, approximateLineCount int32) []lin
 	column := int32(0)
 
 	// Preallocate the top-level table using the approximate line count from the lexer
-	lineOffsetTables := make([]lineOffsetTable, 0, approximateLineCount)
+	lineOffsetTables := make([]LineOffsetTable, 0, approximateLineCount)
 
 	for i, c := range contents {
 		// Mark the start of the next line
@@ -726,7 +726,7 @@ func generateLineOffsetTables(contents string, approximateLineCount int32) []lin
 				}
 			}
 
-			lineOffsetTables = append(lineOffsetTables, lineOffsetTable{
+			lineOffsetTables = append(lineOffsetTables, LineOffsetTable{
 				byteOffsetToStartOfLine:   int32(lineByteOffset),
 				byteOffsetToFirstNonASCII: byteOffsetToFirstNonASCII,
 				columnsForNonASCII:        columnsForNonASCII,
@@ -758,7 +758,7 @@ func generateLineOffsetTables(contents string, approximateLineCount int32) []lin
 		}
 	}
 
-	lineOffsetTables = append(lineOffsetTables, lineOffsetTable{
+	lineOffsetTables = append(lineOffsetTables, LineOffsetTable{
 		byteOffsetToStartOfLine:   int32(lineByteOffset),
 		byteOffsetToFirstNonASCII: byteOffsetToFirstNonASCII,
 		columnsForNonASCII:        columnsForNonASCII,
@@ -810,9 +810,9 @@ func (p *printer) printSymbol(ref js_ast.Ref) {
 	p.printIdentifier(p.renamer.NameForSymbol(ref))
 }
 
-func CanQuoteIdentifier(name string, options *config.Options) bool {
-	return js_lexer.IsIdentifier(name) && (!options.ASCIIOnly ||
-		!options.UnsupportedJSFeatures.Has(compat.UnicodeEscapes) ||
+func CanQuoteIdentifier(name string, unsupportedJSFeatures compat.JSFeature, asciiOnly bool) bool {
+	return js_lexer.IsIdentifier(name) && (!asciiOnly ||
+		!unsupportedJSFeatures.Has(compat.UnicodeEscapes) ||
 		!js_lexer.ContainsNonBMPCodePoint(name))
 }
 
@@ -1363,10 +1363,20 @@ func (p *printer) printRequireOrImportExpr(importRecordIndex uint32, leadingInte
 
 	// Make sure "import()" expressions return promises
 	if record.Kind == ast.ImportDynamic {
-		if p.options.RemoveWhitespace {
-			p.print("Promise.resolve().then(()=>")
+		if p.options.UnsupportedFeatures.Has(compat.Arrow) {
+			p.print("Promise.resolve().then(function()")
+			p.printSpace()
+			p.print("{")
+			p.printNewline()
+			p.options.Indent++
+			p.printIndent()
+			p.print("return ")
 		} else {
-			p.print("Promise.resolve().then(() => ")
+			if p.options.RemoveWhitespace {
+				p.print("Promise.resolve().then(()=>")
+			} else {
+				p.print("Promise.resolve().then(() => ")
+			}
 		}
 	}
 
@@ -1394,7 +1404,17 @@ func (p *printer) printRequireOrImportExpr(importRecordIndex uint32, leadingInte
 	}
 
 	if record.Kind == ast.ImportDynamic {
-		p.print(")")
+		if p.options.UnsupportedFeatures.Has(compat.Arrow) {
+			if !p.options.RemoveWhitespace {
+				p.print(";")
+			}
+			p.printNewline()
+			p.options.Indent--
+			p.printIndent()
+			p.print("})")
+		} else {
+			p.print(")")
+		}
 	}
 }
 
@@ -3052,44 +3072,30 @@ func (p *printer) shouldIgnoreSourceMap() bool {
 	return true
 }
 
-type PrintOptions struct {
+type Options struct {
 	OutputFormat        config.Format
 	RemoveWhitespace    bool
 	MangleSyntax        bool
 	ASCIIOnly           bool
 	ExtractComments     bool
+	AddSourceMappings   bool
 	Indent              int
 	ToModuleRef         js_ast.Ref
 	WrapperRefForSource func(uint32) js_ast.Ref
 	UnsupportedFeatures compat.JSFeature
 
-	// This contains the contents of the input file to map back to in the source
-	// map. If it's nil that means we're not generating source maps.
-	SourceForSourceMap *logger.Source
+	// If we're writing out a source map, this table of line start indices lets
+	// us do binary search on to figure out what line a given AST node came from
+	LineOffsetTables []LineOffsetTable
 
 	// This will be present if the input file had a source map. In that case we
 	// want to map all the way back to the original input file(s).
 	InputSourceMap *sourcemap.SourceMap
 }
 
-type QuotedSource struct {
-	// These are quoted ahead of time instead of during source map generation so
-	// the quoting happens in parallel instead of in serial
-	QuotedPath     []byte
-	QuotedContents []byte
-}
-
 type SourceMapChunk struct {
 	Buffer []byte
 
-	// There may be more than one source for this chunk if the file being printed
-	// has an associated source map. In that case the "source index" values in
-	// the buffer are 0-based indices into this array. The source index of the
-	// first mapping will be adjusted when the chunks are joined together. Since
-	// the source indices are encoded using a delta from the previous source
-	// index, none of the other source indices need to be modified while joining.
-	QuotedSources []QuotedSource
-
 	// This end state will be used to rewrite the start of the following source
 	// map chunk so that the delta-encoded VLQ numbers are preserved.
 	EndState SourceMapState
@@ -3106,49 +3112,6 @@ type SourceMapChunk struct {
 	ShouldIgnore bool
 }
 
-func createPrinter(
-	symbols js_ast.SymbolMap,
-	r renamer.Renamer,
-	importRecords []ast.ImportRecord,
-	options PrintOptions,
-	approximateLineCount int32,
-) *printer {
-	p := &printer{
-		symbols:            symbols,
-		renamer:            r,
-		importRecords:      importRecords,
-		options:            options,
-		stmtStart:          -1,
-		exportDefaultStart: -1,
-		arrowExprStart:     -1,
-		prevOpEnd:          -1,
-		prevNumEnd:         -1,
-		prevRegExpEnd:      -1,
-		prevLoc:            logger.Loc{Start: -1},
-
-		// We automatically repeat the previous source mapping if we ever generate
-		// a line that doesn't start with a mapping. This helps give files more
-		// complete mapping coverage without gaps.
-		//
-		// However, we probably shouldn't do this if the input file has a nested
-		// source map that we will be remapping through. We have no idea what state
-		// that source map is in and it could be pretty scrambled.
-		//
-		// I've seen cases where blindly repeating the last mapping for subsequent
-		// lines gives very strange and unhelpful results with source maps from
-		// other tools.
-		coverLinesWithoutMappings: options.InputSourceMap == nil,
-	}
-
-	// If we're writing out a source map, prepare a table of line start indices
-	// to do binary search on to figure out what line a given AST node came from
-	if options.SourceForSourceMap != nil {
-		p.lineOffsetTables = generateLineOffsetTables(options.SourceForSourceMap.Contents, approximateLineCount)
-	}
-
-	return p
-}
-
 type PrintResult struct {
 	JS []byte
 
@@ -3164,8 +3127,34 @@ type PrintResult struct {
 	FirstDeclSourceMapOffset uint32
 }
 
-func Print(tree js_ast.AST, symbols js_ast.SymbolMap, r renamer.Renamer, options PrintOptions) PrintResult {
-	p := createPrinter(symbols, r, tree.ImportRecords, options, tree.ApproximateLineCount)
+func Print(tree js_ast.AST, symbols js_ast.SymbolMap, r renamer.Renamer, options Options) PrintResult {
+	p := &printer{
+		symbols:            symbols,
+		renamer:            r,
+		importRecords:      tree.ImportRecords,
+		options:            options,
+		stmtStart:          -1,
+		exportDefaultStart: -1,
+		arrowExprStart:     -1,
+		prevOpEnd:          -1,
+		prevNumEnd:         -1,
+		prevRegExpEnd:      -1,
+		prevLoc:            logger.Loc{Start: -1},
+		lineOffsetTables:   options.LineOffsetTables,
+
+		// We automatically repeat the previous source mapping if we ever generate
+		// a line that doesn't start with a mapping. This helps give files more
+		// complete mapping coverage without gaps.
+		//
+		// However, we probably shouldn't do this if the input file has a nested
+		// source map that we will be remapping through. We have no idea what state
+		// that source map is in and it could be pretty scrambled.
+		//
+		// I've seen cases where blindly repeating the last mapping for subsequent
+		// lines gives very strange and unhelpful results with source maps from
+		// other tools.
+		coverLinesWithoutMappings: options.InputSourceMap == nil,
+	}
 
 	for _, part := range tree.Parts {
 		for _, stmt := range part.Stmts {
@@ -3181,7 +3170,6 @@ func Print(tree js_ast.AST, symbols js_ast.SymbolMap, r renamer.Renamer, options
 		ExtractedComments: p.extractedComments,
 		SourceMapChunk: SourceMapChunk{
 			Buffer:               p.sourceMap,
-			QuotedSources:        quotedSources(&tree, &options),
 			EndState:             p.prevState,
 			FinalGeneratedColumn: p.generatedColumn,
 			FinalLocalSemi:       p.finalLocalSemi,
@@ -3191,51 +3179,3 @@ func Print(tree js_ast.AST, symbols js_ast.SymbolMap, r renamer.Renamer, options
 		FirstDeclSourceMapOffset: p.firstDeclSourceMapOffset,
 	}
 }
-
-func PrintExpr(expr js_ast.Expr, symbols js_ast.SymbolMap, r renamer.Renamer, options PrintOptions) PrintResult {
-	p := createPrinter(symbols, r, nil, options, 0)
-
-	p.printExpr(expr, js_ast.LLowest, 0)
-
-	p.updateGeneratedLineAndColumn()
-
-	return PrintResult{
-		JS:                p.js,
-		ExtractedComments: p.extractedComments,
-		SourceMapChunk: SourceMapChunk{
-			Buffer:               p.sourceMap,
-			QuotedSources:        quotedSources(nil, &options),
-			EndState:             p.prevState,
-			FinalGeneratedColumn: p.generatedColumn,
-			ShouldIgnore:         p.shouldIgnoreSourceMap(),
-		},
-	}
-}
-
-func quotedSources(tree *js_ast.AST, options *PrintOptions) []QuotedSource {
-	if options.SourceForSourceMap == nil {
-		return nil
-	}
-
-	if sm := options.InputSourceMap; sm != nil {
-		results := make([]QuotedSource, len(sm.Sources))
-		for i, source := range sm.Sources {
-			contents := []byte("null")
-			if i < len(sm.SourcesContent) {
-				if value := sm.SourcesContent[i]; value != nil {
-					contents = QuoteForJSON(*value, options.ASCIIOnly)
-				}
-			}
-			results[i] = QuotedSource{
-				QuotedPath:     QuoteForJSON(source, options.ASCIIOnly),
-				QuotedContents: contents,
-			}
-		}
-		return results
-	}
-
-	return []QuotedSource{{
-		QuotedPath:     QuoteForJSON(options.SourceForSourceMap.PrettyPath, options.ASCIIOnly),
-		QuotedContents: QuoteForJSON(options.SourceForSourceMap.Contents, options.ASCIIOnly),
-	}}
-}
