@@ -8,9 +8,11 @@ package logger
 import (
 	"fmt"
 	"os"
+	"runtime"
 	"sort"
 	"strings"
 	"sync"
+	"time"
 )
 
 type Log struct {
@@ -34,10 +36,29 @@ type MsgKind uint8
 const (
 	Error MsgKind = iota
 	Warning
+	Note
 )
 
+func (kind MsgKind) String() string {
+	switch kind {
+	case Error:
+		return "error"
+	case Warning:
+		return "warning"
+	case Note:
+		return "note"
+	default:
+		panic("Internal error")
+	}
+}
+
 type Msg struct {
-	Kind     MsgKind
+	Kind  MsgKind
+	Data  MsgData
+	Notes []MsgData
+}
+
+type MsgData struct {
 	Text     string
 	Location *MsgLocation
 }
@@ -66,70 +87,32 @@ func (r Range) End() int32 {
 }
 
 // This type is just so we can use Go's native sort function
-type msgsArray []Msg
+type SortableMsgs []Msg
 
-func (a msgsArray) Len() int          { return len(a) }
-func (a msgsArray) Swap(i int, j int) { a[i], a[j] = a[j], a[i] }
+func (a SortableMsgs) Len() int          { return len(a) }
+func (a SortableMsgs) Swap(i int, j int) { a[i], a[j] = a[j], a[i] }
 
-func (a msgsArray) Less(i int, j int) bool {
+func (a SortableMsgs) Less(i int, j int) bool {
 	ai := a[i]
 	aj := a[j]
-
-	li := ai.Location
-	lj := aj.Location
-
-	// Location
-	if li == nil && lj != nil {
-		return true
+	aiLoc := ai.Data.Location
+	ajLoc := aj.Data.Location
+	if aiLoc == nil || ajLoc == nil {
+		return aiLoc == nil && ajLoc != nil
 	}
-	if li != nil && lj == nil {
-		return false
+	if aiLoc.File != ajLoc.File {
+		return aiLoc.File < ajLoc.File
 	}
-
-	if li != nil && lj != nil {
-		// File
-		if li.File < lj.File {
-			return true
-		}
-		if li.File > lj.File {
-			return false
-		}
-
-		// Line
-		if li.Line < lj.Line {
-			return true
-		}
-		if li.Line > lj.Line {
-			return false
-		}
-
-		// Column
-		if li.Column < lj.Column {
-			return true
-		}
-		if li.Column > lj.Column {
-			return false
-		}
-
-		// Length
-		if li.Length < lj.Length {
-			return true
-		}
-		if li.Length > lj.Length {
-			return false
-		}
+	if aiLoc.Line != ajLoc.Line {
+		return aiLoc.Line < ajLoc.Line
 	}
-
-	// Kind
-	if ai.Kind < aj.Kind {
-		return true
+	if aiLoc.Column != ajLoc.Column {
+		return aiLoc.Column < ajLoc.Column
 	}
-	if ai.Kind > aj.Kind {
-		return false
+	if ai.Kind != aj.Kind {
+		return ai.Kind < aj.Kind
 	}
-
-	// Text
-	return ai.Text < aj.Text
+	return ai.Data.Text < aj.Data.Text
 }
 
 // This is used to represent both file system paths (Namespace == "file") and
@@ -145,6 +128,37 @@ func (a Path) ComesBeforeInSortedOrder(b Path) bool {
 	return a.Namespace > b.Namespace || (a.Namespace == b.Namespace && a.Text < b.Text)
 }
 
+// This has a custom implementation instead of using "filepath.Dir/Base/Ext"
+// because it should work the same on Unix and Windows. These names end up in
+// the generated output and the generated output should not depend on the OS.
+func PlatformIndependentPathDirBaseExt(path string) (dir string, base string, ext string) {
+	for {
+		i := strings.LastIndexAny(path, "/\\")
+
+		// Stop if there are no more slashes
+		if i < 0 {
+			base = path
+			break
+		}
+
+		// Stop if we found a non-trailing slash
+		if i+1 != len(path) {
+			dir, base = path[:i], path[i+1:]
+			break
+		}
+
+		// Ignore trailing slashes
+		path = path[:i]
+	}
+
+	// Strip off the extension
+	if dot := strings.LastIndexByte(base, '.'); dot >= 0 {
+		base, ext = base[:dot], base[dot:]
+	}
+
+	return
+}
+
 type Source struct {
 	Index uint32
 
@@ -267,9 +281,9 @@ type TerminalInfo struct {
 	Width           int
 }
 
-func NewStderrLog(options StderrOptions) Log {
+func NewStderrLog(options OutputOptions) Log {
 	var mutex sync.Mutex
-	var msgs msgsArray
+	var msgs SortableMsgs
 	terminalInfo := GetTerminalInfo(os.Stderr)
 	errors := 0
 	warnings := 0
@@ -336,11 +350,11 @@ func NewStderrLog(options StderrOptions) Log {
 }
 
 func PrintErrorToStderr(osArgs []string, text string) {
-	PrintMessageToStderr(osArgs, Msg{Kind: Error, Text: text})
+	PrintMessageToStderr(osArgs, Msg{Kind: Error, Data: MsgData{Text: text}})
 }
 
-func PrintMessageToStderr(osArgs []string, msg Msg) {
-	options := StderrOptions{IncludeSource: true}
+func OutputOptionsForArgs(osArgs []string) OutputOptions {
+	options := OutputOptions{IncludeSource: true}
 
 	// Implement a mini argument parser so these options always work even if we
 	// haven't yet gotten to the general-purpose argument parsing code
@@ -361,13 +375,261 @@ func PrintMessageToStderr(osArgs []string, msg Msg) {
 		}
 	}
 
-	log := NewStderrLog(options)
+	return options
+}
+
+func PrintMessageToStderr(osArgs []string, msg Msg) {
+	log := NewStderrLog(OutputOptionsForArgs(osArgs))
 	log.AddMsg(msg)
 	log.Done()
 }
 
+type Colors struct {
+	Default string
+	Bold    string
+	Dim     string
+
+	Red   string
+	Green string
+	Blue  string
+
+	Cyan    string
+	Magenta string
+	Yellow  string
+
+	Underline string
+}
+
+func PrintText(file *os.File, level LogLevel, osArgs []string, callback func(Colors) string) {
+	options := OutputOptionsForArgs(osArgs)
+
+	// Skip logging these if these logs are disabled
+	if options.LogLevel > level {
+		return
+	}
+
+	var useColorEscapes bool
+	switch options.Color {
+	case ColorNever:
+		useColorEscapes = false
+	case ColorAlways:
+		useColorEscapes = SupportsColorEscapes
+	case ColorIfTerminal:
+		useColorEscapes = GetTerminalInfo(file).UseColorEscapes
+	}
+
+	var colors Colors
+	if useColorEscapes {
+		colors.Default = colorReset
+		colors.Bold = colorResetBold
+		colors.Dim = colorResetDim
+
+		colors.Red = colorRed
+		colors.Green = colorGreen
+		colors.Blue = colorBlue
+
+		colors.Cyan = colorCyan
+		colors.Magenta = colorMagenta
+		colors.Yellow = colorYellow
+
+		colors.Underline = colorResetUnderline
+	}
+	writeStringWithColor(file, callback(colors))
+}
+
+type SummaryTableEntry struct {
+	Dir         string
+	Base        string
+	Size        string
+	Bytes       int
+	IsSourceMap bool
+}
+
+// This type is just so we can use Go's native sort function
+type SummaryTable []SummaryTableEntry
+
+func (t SummaryTable) Len() int          { return len(t) }
+func (t SummaryTable) Swap(i int, j int) { t[i], t[j] = t[j], t[i] }
+
+func (t SummaryTable) Less(i int, j int) bool {
+	ti := t[i]
+	tj := t[j]
+
+	// Sort source maps last
+	if !ti.IsSourceMap && tj.IsSourceMap {
+		return true
+	}
+	if ti.IsSourceMap && !tj.IsSourceMap {
+		return false
+	}
+
+	// Sort by size first
+	if ti.Bytes > tj.Bytes {
+		return true
+	}
+	if ti.Bytes < tj.Bytes {
+		return false
+	}
+
+	// Sort subdirectories first
+	if strings.HasPrefix(ti.Dir, tj.Dir) {
+		return true
+	}
+	if strings.HasPrefix(tj.Dir, ti.Dir) {
+		return false
+	}
+
+	// Sort alphabetically by directory first
+	if ti.Dir < tj.Dir {
+		return true
+	}
+	if ti.Dir > tj.Dir {
+		return false
+	}
+
+	// Then sort alphabetically by file name
+	return ti.Base < tj.Base
+}
+
+// Show a warning icon next to output files that are 1mb or larger
+const sizeWarningThreshold = 1024 * 1024
+
+func PrintSummary(osArgs []string, table SummaryTable, start time.Time) {
+	PrintText(os.Stderr, LevelInfo, osArgs, func(colors Colors) string {
+		isProbablyWindowsCommandPrompt := false
+		sb := strings.Builder{}
+
+		// Assume we are running in Windows Command Prompt if we're on Windows. If
+		// so, we can't use emoji because it won't be supported. Except we can
+		// still use emoji if the WT_SESSION environment variable is present
+		// because that means we're running in the new Windows Terminal instead.
+		if runtime.GOOS == "windows" {
+			isProbablyWindowsCommandPrompt = true
+			for _, env := range os.Environ() {
+				if strings.HasPrefix(env, "WT_SESSION=") {
+					isProbablyWindowsCommandPrompt = false
+					break
+				}
+			}
+		}
+
+		if len(table) > 0 {
+			// Compute the maximum width of the size column
+			spacingBetweenColumns := 2
+			hasSizeWarning := false
+			maxPath := 0
+			maxSize := 0
+			for _, entry := range table {
+				path := len(entry.Dir) + len(entry.Base)
+				size := len(entry.Size) + spacingBetweenColumns
+				if path > maxPath {
+					maxPath = path
+				}
+				if size > maxSize {
+					maxSize = size
+				}
+				if !entry.IsSourceMap && entry.Bytes >= sizeWarningThreshold {
+					hasSizeWarning = true
+				}
+			}
+
+			margin := "  "
+			layoutWidth := GetTerminalInfo(os.Stderr).Width - 2*len(margin)
+			if hasSizeWarning {
+				// Add space for the warning icon
+				layoutWidth -= 2
+			}
+			if layoutWidth > maxPath+maxSize {
+				layoutWidth = maxPath + maxSize
+			}
+			sort.Sort(table)
+			sb.WriteString("\n")
+
+			wasSourceMap := false
+			for i, entry := range table {
+				dir, base := entry.Dir, entry.Base
+				pathWidth := layoutWidth - maxSize
+
+				// Truncate the path with "..." to fit on one line
+				if len(dir)+len(base) > pathWidth {
+					// Trim the directory from the front, leaving the trailing slash
+					if len(dir) > 0 {
+						n := pathWidth - len(base) - 3
+						if n < 1 {
+							n = 1
+						}
+						dir = "..." + dir[len(dir)-n:]
+					}
+
+					// Trim the file name from the back
+					if len(dir)+len(base) > pathWidth {
+						n := pathWidth - len(dir) - 3
+						if n < 0 {
+							n = 0
+						}
+						base = base[:n] + "..."
+					}
+				}
+
+				spacer := layoutWidth - len(entry.Size) - len(dir) - len(base)
+				if spacer < 0 {
+					spacer = 0
+				}
+
+				// Print a boundary in between normal files and source map files if
+				// there was more than one normal file. This improves scannability.
+				if !wasSourceMap && entry.IsSourceMap && i > 1 {
+					sb.WriteString("\n")
+					wasSourceMap = true
+				}
+
+				// Put a warning next to the size if it's above a certain threshold
+				sizeColor := colors.Cyan
+				sizeWarning := ""
+				if !entry.IsSourceMap && entry.Bytes >= sizeWarningThreshold {
+					sizeColor = colors.Yellow
+
+					// Emoji don't work in Windows Command Prompt
+					if !isProbablyWindowsCommandPrompt {
+						sizeWarning = " ⚠️"
+					}
+				}
+
+				sb.WriteString(fmt.Sprintf("%s%s%s%s%s%s%s%s%s%s%s\n",
+					margin,
+					colors.Dim,
+					dir,
+					colors.Bold,
+					base,
+					colors.Default,
+					strings.Repeat(" ", spacer),
+					sizeColor,
+					entry.Size,
+					sizeWarning,
+					colors.Default,
+				))
+			}
+		}
+
+		lightningSymbol := "⚡ "
+
+		// Emoji don't work in Windows Command Prompt
+		if isProbablyWindowsCommandPrompt {
+			lightningSymbol = ""
+		}
+
+		sb.WriteString(fmt.Sprintf("\n%s%sDone in %dms%s\n\n",
+			lightningSymbol,
+			colors.Green,
+			time.Since(start).Milliseconds(),
+			colors.Default,
+		))
+		return sb.String()
+	})
+}
+
 func NewDeferLog() Log {
-	var msgs msgsArray
+	var msgs SortableMsgs
 	var mutex sync.Mutex
 	var hasErrors bool
 
@@ -395,86 +657,154 @@ func NewDeferLog() Log {
 }
 
 const colorReset = "\033[0m"
+
 const colorRed = "\033[31m"
 const colorGreen = "\033[32m"
+const colorBlue = "\033[34m"
+
+const colorCyan = "\033[36m"
 const colorMagenta = "\033[35m"
+const colorYellow = "\033[33m"
+
+const colorResetDim = "\033[0;37m"
 const colorBold = "\033[1m"
 const colorResetBold = "\033[0;1m"
+const colorResetUnderline = "\033[0;4m"
 
-type StderrColor uint8
+type UseColor uint8
 
 const (
-	ColorIfTerminal StderrColor = iota
+	ColorIfTerminal UseColor = iota
 	ColorNever
 	ColorAlways
 )
 
-type StderrOptions struct {
+type OutputOptions struct {
 	IncludeSource bool
 	ErrorLimit    int
-	Color         StderrColor
+	Color         UseColor
 	LogLevel      LogLevel
 }
 
-func (msg Msg) String(options StderrOptions, terminalInfo TerminalInfo) string {
-	kind := "error"
-	kindColor := colorRed
-
-	if msg.Kind == Warning {
-		kind = "warning"
-		kindColor = colorMagenta
+func (msg Msg) String(options OutputOptions, terminalInfo TerminalInfo) string {
+	// Compute the maximum margin
+	maxMargin := 0
+	if options.IncludeSource {
+		if msg.Data.Location != nil {
+			maxMargin = len(fmt.Sprintf("%d", msg.Data.Location.Line))
+		}
+		for _, note := range msg.Notes {
+			if note.Location != nil {
+				margin := len(fmt.Sprintf("%d", note.Location.Line))
+				if margin > maxMargin {
+					maxMargin = margin
+				}
+			}
+		}
 	}
 
-	if msg.Location == nil {
+	// Format the messages
+	text := msgString(options, terminalInfo, msg.Kind, msg.Data, maxMargin)
+	for _, note := range msg.Notes {
+		text += msgString(options, terminalInfo, Note, note, maxMargin)
+	}
+
+	// Add extra spacing between messages if source code is present
+	if options.IncludeSource {
+		text += "\n"
+	}
+	return text
+}
+
+// The number of margin characters in addition to the line number
+const extraMarginChars = 7
+
+func marginWithLineText(maxMargin int, line int) string {
+	number := fmt.Sprintf("%d", line)
+	return fmt.Sprintf("    %s%s │ ", strings.Repeat(" ", maxMargin-len(number)), number)
+}
+
+func emptyMarginText(maxMargin int) string {
+	return fmt.Sprintf("    %s ╵ ", strings.Repeat(" ", maxMargin))
+}
+
+func msgString(options OutputOptions, terminalInfo TerminalInfo, kind MsgKind, data MsgData, maxMargin int) string {
+	var kindColor string
+	textColor := colorBold
+	textResetColor := colorResetBold
+	textIndent := ""
+
+	if options.IncludeSource {
+		textIndent = " > "
+	}
+
+	switch kind {
+	case Error:
+		kindColor = colorRed
+
+	case Warning:
+		kindColor = colorMagenta
+
+	case Note:
+		textColor = colorReset
+		kindColor = colorResetBold
+		textResetColor = colorReset
+		if options.IncludeSource {
+			textIndent = strings.Repeat(" ", maxMargin+extraMarginChars-3)
+		}
+
+	default:
+		panic("Internal error")
+	}
+
+	if data.Location == nil {
 		if terminalInfo.UseColorEscapes {
-			return fmt.Sprintf("%s%s%s: %s%s%s\n",
-				colorBold, kindColor, kind,
-				colorResetBold, msg.Text,
+			return fmt.Sprintf("%s%s%s%s: %s%s%s\n",
+				textColor, textIndent, kindColor, kind.String(),
+				textResetColor, data.Text,
 				colorReset)
 		}
 
-		return fmt.Sprintf("%s: %s\n", kind, msg.Text)
+		return fmt.Sprintf("%s%s: %s\n", textIndent, kind.String(), data.Text)
 	}
 
 	if !options.IncludeSource {
 		if terminalInfo.UseColorEscapes {
-			return fmt.Sprintf("%s%s: %s%s: %s%s%s\n",
-				colorBold, msg.Location.File,
-				kindColor, kind,
-				colorResetBold, msg.Text,
+			return fmt.Sprintf("%s%s%s: %s%s: %s%s%s\n",
+				textColor, textIndent, data.Location.File,
+				kindColor, kind.String(),
+				textResetColor, data.Text,
 				colorReset)
 		}
 
-		return fmt.Sprintf("%s: %s: %s\n", msg.Location.File, kind, msg.Text)
+		return fmt.Sprintf("%s%s: %s: %s\n",
+			textIndent, data.Location.File, kind.String(), data.Text)
 	}
 
-	d := detailStruct(msg, terminalInfo)
+	d := detailStruct(data, terminalInfo, maxMargin)
 
 	if terminalInfo.UseColorEscapes {
-		return fmt.Sprintf("%s%s:%d:%d: %s%s: %s%s\n%s%s%s%s%s%s\n%s%s%s%s%s\n",
-			colorBold, d.Path,
-			d.Line,
-			d.Column,
-			kindColor, d.Kind,
-			colorResetBold, d.Message,
-			colorReset, d.SourceBefore, colorGreen, d.SourceMarked, colorReset, d.SourceAfter,
-			colorGreen, d.Indent, d.Marker,
-			colorReset, d.ContentAfter)
+		return fmt.Sprintf("%s%s%s: %s%s: %s%s\n%s%s%s%s%s%s\n%s%s%s%s%s%s\n",
+			textColor, textIndent, d.Path,
+			kindColor, kind.String(),
+			textResetColor, d.Message,
+			colorResetDim, d.SourceBefore, colorGreen, d.SourceMarked, colorResetDim, d.SourceAfter,
+			d.Indent, colorGreen, d.Marker,
+			colorResetDim, d.ContentAfter, colorReset)
 	}
 
-	return fmt.Sprintf("%s:%d:%d: %s: %s\n%s\n%s%s%s\n",
-		d.Path, d.Line, d.Column, d.Kind, d.Message, d.Source, d.Indent, d.Marker, d.ContentAfter)
+	return fmt.Sprintf("%s%s: %s: %s\n%s%s%s\n%s%s%s\n",
+		textIndent, d.Path, kind.String(), d.Message,
+		d.SourceBefore, d.SourceMarked, d.SourceAfter,
+		d.Indent, d.Marker, d.ContentAfter)
 }
 
 type MsgDetail struct {
 	Path    string
 	Line    int
 	Column  int
-	Kind    string
 	Message string
 
-	// Source == SourceBefore + SourceMarked + SourceAfter
-	Source       string
 	SourceBefore string
 	SourceMarked string
 	SourceAfter  string
@@ -541,9 +871,9 @@ func LocationOrNil(source *Source, r Range) *MsgLocation {
 	}
 }
 
-func detailStruct(msg Msg, terminalInfo TerminalInfo) MsgDetail {
+func detailStruct(data MsgData, terminalInfo TerminalInfo, maxMargin int) MsgDetail {
 	// Only highlight the first line of the line text
-	loc := *msg.Location
+	loc := *data.Location
 	endOfFirstLine := len(loc.LineText)
 	for i, c := range loc.LineText {
 		if c == '\r' || c == '\n' || c == '\u2028' || c == '\u2029' {
@@ -599,6 +929,10 @@ func detailStruct(msg Msg, terminalInfo TerminalInfo) MsgDetail {
 	if width < 1 {
 		width = 80
 	}
+	width -= maxMargin + extraMarginChars
+	if width < 1 {
+		width = 1
+	}
 	if loc.Column == endOfFirstLine {
 		// If the marker is at the very end of the line, the marker will be a "^"
 		// character that extends one column past the end of the line. In this case
@@ -657,24 +991,20 @@ func detailStruct(msg Msg, terminalInfo TerminalInfo) MsgDetail {
 		marker = strings.Repeat("~", markerEnd-markerStart)
 	}
 
-	kind := "error"
-	if msg.Kind == Warning {
-		kind = "warning"
-	}
+	// Put a margin before the marker indent
+	margin := marginWithLineText(maxMargin, loc.Line)
 
 	return MsgDetail{
 		Path:    loc.File,
 		Line:    loc.Line,
 		Column:  loc.Column,
-		Kind:    kind,
-		Message: msg.Text,
+		Message: data.Text,
 
-		Source:       lineText,
-		SourceBefore: lineText[:markerStart],
+		SourceBefore: margin + lineText[:markerStart],
 		SourceMarked: lineText[markerStart:markerEnd],
 		SourceAfter:  lineText[markerEnd:],
 
-		Indent: indent,
+		Indent: emptyMarginText(maxMargin) + indent,
 		Marker: marker,
 
 		ContentAfter: afterFirstLine,
@@ -707,32 +1037,59 @@ func renderTabStops(withTabs string, spacesPerTab int) string {
 
 func (log Log) AddError(source *Source, loc Loc, text string) {
 	log.AddMsg(Msg{
-		Kind:     Error,
-		Text:     text,
-		Location: LocationOrNil(source, Range{Loc: loc}),
+		Kind: Error,
+		Data: RangeData(source, Range{Loc: loc}, text),
+	})
+}
+
+func (log Log) AddErrorWithNotes(source *Source, loc Loc, text string, notes []MsgData) {
+	log.AddMsg(Msg{
+		Kind:  Error,
+		Data:  RangeData(source, Range{Loc: loc}, text),
+		Notes: notes,
 	})
 }
 
 func (log Log) AddWarning(source *Source, loc Loc, text string) {
 	log.AddMsg(Msg{
-		Kind:     Warning,
-		Text:     text,
-		Location: LocationOrNil(source, Range{Loc: loc}),
+		Kind: Warning,
+		Data: RangeData(source, Range{Loc: loc}, text),
 	})
 }
 
 func (log Log) AddRangeError(source *Source, r Range, text string) {
 	log.AddMsg(Msg{
-		Kind:     Error,
-		Text:     text,
-		Location: LocationOrNil(source, r),
+		Kind: Error,
+		Data: RangeData(source, r, text),
 	})
 }
 
 func (log Log) AddRangeWarning(source *Source, r Range, text string) {
 	log.AddMsg(Msg{
-		Kind:     Warning,
-		Text:     text,
-		Location: LocationOrNil(source, r),
+		Kind: Warning,
+		Data: RangeData(source, r, text),
 	})
 }
+
+func (log Log) AddRangeErrorWithNotes(source *Source, r Range, text string, notes []MsgData) {
+	log.AddMsg(Msg{
+		Kind:  Error,
+		Data:  RangeData(source, r, text),
+		Notes: notes,
+	})
+}
+
+func (log Log) AddRangeWarningWithNotes(source *Source, r Range, text string, notes []MsgData) {
+	log.AddMsg(Msg{
+		Kind:  Warning,
+		Data:  RangeData(source, r, text),
+		Notes: notes,
+	})
+}
+
+func RangeData(source *Source, r Range, text string) MsgData {
+	return MsgData{
+		Text:     text,
+		Location: LocationOrNil(source, r),
+	}
+}
