@@ -3,11 +3,13 @@ package resolver
 import (
 	"errors"
 	"fmt"
+	"regexp"
 	"strings"
 	"sync"
 	"syscall"
 
 	"github.com/evanw/esbuild/internal/ast"
+	"github.com/evanw/esbuild/internal/cache"
 	"github.com/evanw/esbuild/internal/config"
 	"github.com/evanw/esbuild/internal/fs"
 	"github.com/evanw/esbuild/internal/js_ast"
@@ -74,17 +76,26 @@ func (pp *PathPair) HasSecondary() bool {
 	return pp.Secondary.Text != ""
 }
 
+type IgnoreIfUnusedData struct {
+	Source *logger.Source
+	Range  logger.Range
+
+	// If true, "sideEffects" was an array. If false, "sideEffects" was false.
+	IsSideEffectsArrayInJSON bool
+}
+
 type ResolveResult struct {
-	PathPair   PathPair
-	IsExternal bool
+	PathPair PathPair
 
 	// If not empty, these should override the default values
 	JSXFactory  []string // Default if empty: "React.createElement"
 	JSXFragment []string // Default if empty: "React.Fragment"
 
+	IsExternal bool
+
 	// If true, any ES6 imports to this file can be considered to have no side
 	// effects. This means they should be removed if unused.
-	IgnoreIfUnused bool
+	IgnorePrimaryIfUnused *IgnoreIfUnusedData
 
 	// If true, the class field transform should use Object.defineProperty().
 	UseDefineForClassFieldsTS bool
@@ -102,11 +113,16 @@ type Resolver interface {
 	Resolve(sourceDir string, importPath string, kind ast.ImportKind) *ResolveResult
 	ResolveAbs(absPath string) *ResolveResult
 	PrettyPath(path logger.Path) string
+
+	// This tries to run "Resolve" on a package path as a relative path. If
+	// successful, the user just forgot a leading "./" in front of the path.
+	ProbeResolvePackageAsRelative(sourceDir string, importPath string, kind ast.ImportKind) *ResolveResult
 }
 
 type resolver struct {
 	fs      fs.FS
 	log     logger.Log
+	caches  *cache.CacheSet
 	options config.Options
 	mutex   sync.Mutex
 
@@ -139,7 +155,7 @@ type resolver struct {
 	dirCache map[string]*dirInfo
 }
 
-func NewResolver(fs fs.FS, log logger.Log, options config.Options) Resolver {
+func NewResolver(fs fs.FS, log logger.Log, caches *cache.CacheSet, options config.Options) Resolver {
 	// Bundling for node implies allowing node's builtin modules
 	if options.Platform == config.PlatformNode {
 		externalNodeModules := make(map[string]bool)
@@ -167,6 +183,7 @@ func NewResolver(fs fs.FS, log logger.Log, options config.Options) Resolver {
 		fs:                     fs,
 		log:                    log,
 		options:                options,
+		caches:                 caches,
 		dirCache:               make(map[string]*dirInfo),
 		atImportExtensionOrder: atImportExtensionOrder,
 	}
@@ -174,9 +191,10 @@ func NewResolver(fs fs.FS, log logger.Log, options config.Options) Resolver {
 
 func (r *resolver) Resolve(sourceDir string, importPath string, kind ast.ImportKind) *ResolveResult {
 	// Certain types of URLs default to being external for convenience
-	if
-	// "fill: url(#filter);"
-	(kind.IsFromCSS() && strings.HasPrefix(importPath, "#")) ||
+	if r.isExternalPattern(importPath) ||
+
+		// "fill: url(#filter);"
+		(kind.IsFromCSS() && strings.HasPrefix(importPath, "#")) ||
 
 		// "background: url(http://example.com/images/image.png);"
 		strings.HasPrefix(importPath, "http://") ||
@@ -196,6 +214,12 @@ func (r *resolver) Resolve(sourceDir string, importPath string, kind ast.ImportK
 		}
 	}
 
+	// Fail now if there is no directory to resolve in. This can happen for
+	// virtual modules (e.g. stdin) if a resolve directory is not specified.
+	if sourceDir == "" {
+		return nil
+	}
+
 	r.mutex.Lock()
 	defer r.mutex.Unlock()
 
@@ -208,6 +232,17 @@ func (r *resolver) Resolve(sourceDir string, importPath string, kind ast.ImportK
 	return r.finalizeResolve(*result)
 }
 
+func (r *resolver) isExternalPattern(path string) bool {
+	for _, pattern := range r.options.ExternalModules.Patterns {
+		if len(path) >= len(pattern.Prefix)+len(pattern.Suffix) &&
+			strings.HasPrefix(path, pattern.Prefix) &&
+			strings.HasSuffix(path, pattern.Suffix) {
+			return true
+		}
+	}
+	return false
+}
+
 func (r *resolver) ResolveAbs(absPath string) *ResolveResult {
 	r.mutex.Lock()
 	defer r.mutex.Unlock()
@@ -216,6 +251,18 @@ func (r *resolver) ResolveAbs(absPath string) *ResolveResult {
 	return r.finalizeResolve(ResolveResult{PathPair: PathPair{Primary: logger.Path{Text: absPath, Namespace: "file"}}})
 }
 
+func (r *resolver) ProbeResolvePackageAsRelative(sourceDir string, importPath string, kind ast.ImportKind) *ResolveResult {
+	absPath := r.fs.Join(sourceDir, importPath)
+
+	r.mutex.Lock()
+	defer r.mutex.Unlock()
+
+	if pair, ok := r.loadAsFileOrDirectory(absPath, kind); ok {
+		return r.finalizeResolve(ResolveResult{PathPair: pair})
+	}
+	return nil
+}
+
 func isInsideNodeModules(fs fs.FS, path string) bool {
 	dir := fs.Dir(path)
 	for {
@@ -242,13 +289,37 @@ func (r *resolver) finalizeResolve(result ResolveResult) *ResolveResult {
 				}
 
 				// Look up this file in the "sideEffects" map in the nearest enclosing
-				// directory with a "package.json" file
-				for info := dirInfo; info != nil; info = info.parent {
-					if info.packageJSON != nil {
-						if info.packageJSON.sideEffectsMap != nil {
-							result.IgnoreIfUnused = !info.packageJSON.sideEffectsMap[path.Text]
+				// directory with a "package.json" file.
+				//
+				// Only do this for the primary path. Some packages have the primary
+				// path marked as having side effects and the secondary path marked
+				// as not having side effects. This is likely a bug in the package
+				// definition but we don't want to consider the primary path as not
+				// having side effects just because the secondary path is marked as
+				// not having side effects.
+				if *path == result.PathPair.Primary {
+					for info := dirInfo; info != nil; info = info.parent {
+						if info.packageJSON != nil {
+							if info.packageJSON.sideEffectsMap != nil {
+								hasSideEffects := false
+								if info.packageJSON.sideEffectsMap[path.Text] {
+									// Fast path: map lookup
+									hasSideEffects = true
+								} else {
+									// Slow path: glob tests
+									for _, re := range info.packageJSON.sideEffectsRegexps {
+										if re.MatchString(path.Text) {
+											hasSideEffects = true
+											break
+										}
+									}
+								}
+								if !hasSideEffects {
+									result.IgnorePrimaryIfUnused = info.packageJSON.ignoreIfUnusedData
+								}
+							}
+							break
 						}
-						break
 					}
 				}
 
@@ -284,8 +355,7 @@ func (r *resolver) resolveWithoutSymlinks(sourceDir string, importPath string, k
 	// Return early if this is already an absolute path
 	if r.fs.IsAbs(importPath) {
 		// First, check path overrides from the nearest enclosing TypeScript "tsconfig.json" file
-		if dirInfo := r.dirInfoCached(sourceDir); dirInfo != nil && dirInfo.tsConfigJSON != nil &&
-			dirInfo.tsConfigJSON.BaseURL != nil && dirInfo.tsConfigJSON.Paths != nil {
+		if dirInfo := r.dirInfoCached(sourceDir); dirInfo != nil && dirInfo.tsConfigJSON != nil && dirInfo.tsConfigJSON.Paths != nil {
 			if absolute, ok := r.matchTSConfigPaths(dirInfo.tsConfigJSON, importPath, kind); ok {
 				return &ResolveResult{PathPair: absolute}
 			}
@@ -468,7 +538,9 @@ type packageJSON struct {
 	// free of side effects must be included. This convention does not say
 	// anything about whether any statements within the file have side effects or
 	// not.
-	sideEffectsMap map[string]bool
+	sideEffectsMap     map[string]bool
+	sideEffectsRegexps []*regexp.Regexp
+	ignoreIfUnusedData *IgnoreIfUnusedData
 }
 
 type dirInfo struct {
@@ -523,7 +595,7 @@ func (r *resolver) parseTSConfig(file string, visited map[string]bool) (*TSConfi
 	}
 	visited[file] = true
 
-	contents, err := r.fs.ReadFile(file)
+	contents, err := r.caches.FSCache.ReadFile(r.fs, file)
 	if err != nil {
 		return nil, err
 	}
@@ -536,7 +608,7 @@ func (r *resolver) parseTSConfig(file string, visited map[string]bool) (*TSConfi
 	}
 	fileDir := r.fs.Dir(file)
 
-	result := ParseTSConfigJSON(r.log, source, func(extends string, extendsRange logger.Range) *TSConfigJSON {
+	result := ParseTSConfigJSON(r.log, source, &r.caches.JSONCache, func(extends string, extendsRange logger.Range) *TSConfigJSON {
 		if IsPackagePath(extends) {
 			// If this is a package path, try to resolve it to a "node_modules"
 			// folder. This doesn't use the normal node module resolution algorithm
@@ -608,10 +680,14 @@ func (r *resolver) parseTSConfig(file string, visited map[string]bool) (*TSConfi
 		return nil, parseErrorAlreadyLogged
 	}
 
-	if result.BaseURL != nil {
+	if result.BaseURL != nil && !r.fs.IsAbs(*result.BaseURL) {
 		*result.BaseURL = r.fs.Join(fileDir, *result.BaseURL)
 	}
 
+	if result.Paths != nil && !r.fs.IsAbs(result.BaseURLForPaths) {
+		result.BaseURLForPaths = r.fs.Join(fileDir, result.BaseURLForPaths)
+	}
+
 	return result, nil
 }
 
@@ -720,7 +796,7 @@ func (r *resolver) dirInfoUncached(path string) *dirInfo {
 
 func (r *resolver) parsePackageJSON(path string) *packageJSON {
 	packageJsonPath := r.fs.Join(path, "package.json")
-	contents, err := r.fs.ReadFile(packageJsonPath)
+	contents, err := r.caches.FSCache.ReadFile(r.fs, packageJsonPath)
 	if err != nil {
 		r.log.AddError(nil, logger.Loc{},
 			fmt.Sprintf("Cannot read file %q: %s",
@@ -735,7 +811,7 @@ func (r *resolver) parsePackageJSON(path string) *packageJSON {
 		Contents:   contents,
 	}
 
-	json, ok := js_parser.ParseJSON(r.log, jsonSource, js_parser.ParseJSONOptions{})
+	json, ok := r.caches.JSONCache.Parse(r.log, jsonSource, js_parser.JSONOptions{})
 	if !ok {
 		return nil
 	}
@@ -832,38 +908,87 @@ func (r *resolver) parsePackageJSON(path string) *packageJSON {
 	}
 
 	// Read the "sideEffects" property
-	if sideEffectsJson, _, ok := getProperty(json, "sideEffects"); ok {
+	if sideEffectsJson, sideEffectsLoc, ok := getProperty(json, "sideEffects"); ok {
 		switch data := sideEffectsJson.Data.(type) {
 		case *js_ast.EBoolean:
 			if !data.Value {
 				// Make an empty map for "sideEffects: false", which indicates all
 				// files in this module can be considered to not have side effects.
 				packageJSON.sideEffectsMap = make(map[string]bool)
+				packageJSON.ignoreIfUnusedData = &IgnoreIfUnusedData{
+					IsSideEffectsArrayInJSON: false,
+					Source:                   &jsonSource,
+					Range:                    jsonSource.RangeOfString(sideEffectsLoc),
+				}
 			}
 
 		case *js_ast.EArray:
 			// The "sideEffects: []" format means all files in this module but not in
 			// the array can be considered to not have side effects.
 			packageJSON.sideEffectsMap = make(map[string]bool)
+			packageJSON.ignoreIfUnusedData = &IgnoreIfUnusedData{
+				IsSideEffectsArrayInJSON: true,
+				Source:                   &jsonSource,
+				Range:                    jsonSource.RangeOfString(sideEffectsLoc),
+			}
 			for _, itemJson := range data.Items {
-				if item, ok := itemJson.Data.(*js_ast.EString); ok && item.Value != nil {
-					absolute := r.fs.Join(path, js_lexer.UTF16ToString(item.Value))
-					packageJSON.sideEffectsMap[absolute] = true
-				} else {
+				item, ok := itemJson.Data.(*js_ast.EString)
+				if !ok || item.Value == nil {
 					r.log.AddWarning(&jsonSource, itemJson.Loc,
 						"Expected string in array for \"sideEffects\"")
+					continue
 				}
+
+				absPattern := r.fs.Join(path, js_lexer.UTF16ToString(item.Value))
+				re, hadWildcard := globToEscapedRegexp(absPattern)
+
+				// Wildcard patterns require more expensive matching
+				if hadWildcard {
+					packageJSON.sideEffectsRegexps = append(packageJSON.sideEffectsRegexps, regexp.MustCompile(re))
+					continue
+				}
+
+				// Normal strings can be matched with a map lookup
+				packageJSON.sideEffectsMap[absPattern] = true
 			}
 
 		default:
 			r.log.AddWarning(&jsonSource, sideEffectsJson.Loc,
-				"Invalid value for \"sideEffects\"")
+				"The value for \"sideEffects\" must be a boolean or an array")
 		}
 	}
 
 	return packageJSON
 }
 
+func globToEscapedRegexp(glob string) (string, bool) {
+	sb := strings.Builder{}
+	sb.WriteByte('^')
+	hadWildcard := false
+
+	for _, c := range glob {
+		switch c {
+		case '\\', '^', '$', '.', '+', '|', '(', ')', '[', ']', '{', '}':
+			sb.WriteByte('\\')
+			sb.WriteRune(c)
+
+		case '*':
+			sb.WriteString(".*")
+			hadWildcard = true
+
+		case '?':
+			sb.WriteByte('.')
+			hadWildcard = true
+
+		default:
+			sb.WriteRune(c)
+		}
+	}
+
+	sb.WriteByte('$')
+	return sb.String(), hadWildcard
+}
+
 func (r *resolver) loadAsFile(path string, extensionOrder []string) (string, bool) {
 	// Read the directory entries once to minimize locking
 	dirPath := r.fs.Dir(path)
@@ -1038,7 +1163,7 @@ func (r *resolver) matchTSConfigPaths(tsConfigJSON *TSConfigJSON, path string, k
 		if key == path {
 			for _, originalPath := range originalPaths {
 				// Load the original path relative to the "baseUrl" from tsconfig.json
-				absoluteOriginalPath := r.fs.Join(*tsConfigJSON.BaseURL, originalPath)
+				absoluteOriginalPath := r.fs.Join(tsConfigJSON.BaseURLForPaths, originalPath)
 				if absolute, ok := r.loadAsFileOrDirectory(absoluteOriginalPath, kind); ok {
 					return absolute, true
 				}
@@ -1088,7 +1213,7 @@ func (r *resolver) matchTSConfigPaths(tsConfigJSON *TSConfigJSON, path string, k
 			originalPath = strings.Replace(originalPath, "*", matchedText, 1)
 
 			// Load the original path relative to the "baseUrl" from tsconfig.json
-			absoluteOriginalPath := r.fs.Join(*tsConfigJSON.BaseURL, originalPath)
+			absoluteOriginalPath := r.fs.Join(tsConfigJSON.BaseURLForPaths, originalPath)
 			if absolute, ok := r.loadAsFileOrDirectory(absoluteOriginalPath, kind); ok {
 				return absolute, true
 			}
@@ -1100,7 +1225,7 @@ func (r *resolver) matchTSConfigPaths(tsConfigJSON *TSConfigJSON, path string, k
 
 func (r *resolver) loadNodeModules(path string, kind ast.ImportKind, dirInfo *dirInfo) (PathPair, bool) {
 	// First, check path overrides from the nearest enclosing TypeScript "tsconfig.json" file
-	if dirInfo.tsConfigJSON != nil && dirInfo.tsConfigJSON.BaseURL != nil {
+	if dirInfo.tsConfigJSON != nil {
 		// Try path substitutions first
 		if dirInfo.tsConfigJSON.Paths != nil {
 			if absolute, ok := r.matchTSConfigPaths(dirInfo.tsConfigJSON, path, kind); ok {
@@ -1109,9 +1234,11 @@ func (r *resolver) loadNodeModules(path string, kind ast.ImportKind, dirInfo *di
 		}
 
 		// Try looking up the path relative to the base URL
-		basePath := r.fs.Join(*dirInfo.tsConfigJSON.BaseURL, path)
-		if absolute, ok := r.loadAsFileOrDirectory(basePath, kind); ok {
-			return absolute, true
+		if dirInfo.tsConfigJSON.BaseURL != nil {
+			basePath := r.fs.Join(*dirInfo.tsConfigJSON.BaseURL, path)
+			if absolute, ok := r.loadAsFileOrDirectory(basePath, kind); ok {
+				return absolute, true
+			}
 		}
 	}
 
