@@ -56,14 +56,25 @@ function getFlag<T, K extends keyof T>(object: T, keys: OptionKeys, key: K, must
   return value;
 }
 
-function checkForInvalidFlags(object: Object, keys: OptionKeys): void {
+function checkForInvalidFlags(object: Object, keys: OptionKeys, where: string): void {
   for (let key in object) {
     if (!(key in keys)) {
-      throw new Error(`Invalid option: "${key}"`);
+      throw new Error(`Invalid option ${where}: "${key}"`);
     }
   }
 }
 
+export function validateServiceOptions(options: types.ServiceOptions): types.ServiceOptions {
+  let keys: OptionKeys = Object.create(null);
+  let wasmURL = getFlag(options, keys, 'wasmURL', mustBeString);
+  let worker = getFlag(options, keys, 'worker', mustBeBoolean);
+  checkForInvalidFlags(options, keys, 'in startService() call');
+  return {
+    wasmURL,
+    worker,
+  };
+}
+
 type CommonOptions = types.BuildOptions | types.TransformOptions;
 
 function pushLogFlags(flags: string[], options: CommonOptions, keys: OptionKeys, isTTY: boolean, logLevelDefault: types.LogLevel): void {
@@ -126,8 +137,20 @@ function pushCommonFlags(flags: string[], options: CommonOptions, keys: OptionKe
   if (footer) flags.push(`--footer=${footer}`);
 }
 
-function flagsForBuildOptions(options: types.BuildOptions, isTTY: boolean, logLevelDefault: types.LogLevel):
-  [string[], boolean, types.Plugin[] | undefined, string | null, string | null, boolean] {
+function flagsForBuildOptions(
+  callName: string,
+  options: types.BuildOptions,
+  isTTY: boolean,
+  logLevelDefault: types.LogLevel,
+  writeDefault: boolean,
+): {
+  flags: string[],
+  write: boolean,
+  plugins: types.Plugin[] | undefined,
+  stdinContents: string | null,
+  stdinResolveDir: string | null,
+  incremental: boolean,
+} {
   let flags: string[] = [];
   let keys: OptionKeys = Object.create(null);
   let stdinContents: string | null = null;
@@ -136,6 +159,7 @@ function flagsForBuildOptions(options: types.BuildOptions, isTTY: boolean, logLe
   pushCommonFlags(flags, options, keys);
 
   let sourcemap = getFlag(options, keys, 'sourcemap', mustBeStringOrBoolean);
+  let sourcesContent = getFlag(options, keys, 'sourcesContent', mustBeBoolean);
   let bundle = getFlag(options, keys, 'bundle', mustBeBoolean);
   let splitting = getFlag(options, keys, 'splitting', mustBeBoolean);
   let metafile = getFlag(options, keys, 'metafile', mustBeString);
@@ -153,12 +177,13 @@ function flagsForBuildOptions(options: types.BuildOptions, isTTY: boolean, logLe
   let inject = getFlag(options, keys, 'inject', mustBeArray);
   let entryPoints = getFlag(options, keys, 'entryPoints', mustBeArray);
   let stdin = getFlag(options, keys, 'stdin', mustBeObject);
-  let write = getFlag(options, keys, 'write', mustBeBoolean) !== false; // Default to true if not specified
+  let write = getFlag(options, keys, 'write', mustBeBoolean) ?? writeDefault; // Default to true if not specified
   let incremental = getFlag(options, keys, 'incremental', mustBeBoolean) === true;
   let plugins = getFlag(options, keys, 'plugins', mustBeArray);
-  checkForInvalidFlags(options, keys);
+  checkForInvalidFlags(options, keys, `in ${callName}() call`);
 
   if (sourcemap) flags.push(`--sourcemap${sourcemap === true ? '' : `=${sourcemap}`}`);
+  if (sourcesContent !== void 0) flags.push(`--sources-content=${sourcesContent}`);
   if (bundle) flags.push('--bundle');
   if (splitting) flags.push('--splitting');
   if (metafile) flags.push(`--metafile=${metafile}`);
@@ -167,20 +192,36 @@ function flagsForBuildOptions(options: types.BuildOptions, isTTY: boolean, logLe
   if (outbase) flags.push(`--outbase=${outbase}`);
   if (platform) flags.push(`--platform=${platform}`);
   if (tsconfig) flags.push(`--tsconfig=${tsconfig}`);
-  if (resolveExtensions) flags.push(`--resolve-extensions=${resolveExtensions.join(',')}`);
+  if (resolveExtensions) {
+    let values: string[] = [];
+    for (let value of resolveExtensions) {
+      value += '';
+      if (value.indexOf(',') >= 0) throw new Error(`Invalid resolve extension: ${value}`);
+      values.push(value);
+    }
+    flags.push(`--resolve-extensions=${values.join(',')}`);
+  }
   if (publicPath) flags.push(`--public-path=${publicPath}`);
-  if (mainFields) flags.push(`--main-fields=${mainFields.join(',')}`);
+  if (mainFields) {
+    let values: string[] = [];
+    for (let value of mainFields) {
+      value += '';
+      if (value.indexOf(',') >= 0) throw new Error(`Invalid main field: ${value}`);
+      values.push(value);
+    }
+    flags.push(`--main-fields=${values.join(',')}`);
+  }
   if (external) for (let name of external) flags.push(`--external:${name}`);
   if (inject) for (let path of inject) flags.push(`--inject:${path}`);
   if (loader) {
     for (let ext in loader) {
-      if (ext.indexOf('=') >= 0) throw new Error(`Invalid extension: ${ext}`);
+      if (ext.indexOf('=') >= 0) throw new Error(`Invalid loader extension: ${ext}`);
       flags.push(`--loader:${ext}=${loader[ext]}`);
     }
   }
   if (outExtension) {
     for (let ext in outExtension) {
-      if (ext.indexOf('=') >= 0) throw new Error(`Invalid extension: ${ext}`);
+      if (ext.indexOf('=') >= 0) throw new Error(`Invalid out extension: ${ext}`);
       flags.push(`--out-extension:${ext}=${outExtension[ext]}`);
     }
   }
@@ -199,7 +240,7 @@ function flagsForBuildOptions(options: types.BuildOptions, isTTY: boolean, logLe
     let resolveDir = getFlag(stdin, stdinKeys, 'resolveDir', mustBeString);
     let sourcefile = getFlag(stdin, stdinKeys, 'sourcefile', mustBeString);
     let loader = getFlag(stdin, stdinKeys, 'loader', mustBeString);
-    checkForInvalidFlags(stdin, stdinKeys);
+    checkForInvalidFlags(stdin, stdinKeys, 'in "stdin" object');
 
     if (sourcefile) flags.push(`--sourcefile=${sourcefile}`);
     if (loader) flags.push(`--loader=${loader}`);
@@ -207,10 +248,15 @@ function flagsForBuildOptions(options: types.BuildOptions, isTTY: boolean, logLe
     stdinContents = contents ? contents + '' : '';
   }
 
-  return [flags, write, plugins, stdinContents, stdinResolveDir, incremental];
+  return { flags, write, plugins, stdinContents, stdinResolveDir, incremental };
 }
 
-function flagsForTransformOptions(options: types.TransformOptions, isTTY: boolean, logLevelDefault: types.LogLevel): string[] {
+function flagsForTransformOptions(
+  callName: string,
+  options: types.TransformOptions,
+  isTTY: boolean,
+  logLevelDefault: types.LogLevel,
+): string[] {
   let flags: string[] = [];
   let keys: OptionKeys = Object.create(null);
   pushLogFlags(flags, options, keys, isTTY, logLevelDefault);
@@ -220,7 +266,7 @@ function flagsForTransformOptions(options: types.TransformOptions, isTTY: boolea
   let tsconfigRaw = getFlag(options, keys, 'tsconfigRaw', mustBeStringOrObject);
   let sourcefile = getFlag(options, keys, 'sourcefile', mustBeString);
   let loader = getFlag(options, keys, 'loader', mustBeString);
-  checkForInvalidFlags(options, keys);
+  checkForInvalidFlags(options, keys, `in ${callName}() call`);
 
   if (sourcemap) flags.push(`--sourcemap=${sourcemap === true ? 'external' : sourcemap}`);
   if (tsconfigRaw) flags.push(`--tsconfig-raw=${typeof tsconfigRaw === 'string' ? tsconfigRaw : JSON.stringify(tsconfigRaw)}`);
@@ -234,6 +280,7 @@ export interface StreamIn {
   writeToStdin: (data: Uint8Array) => void;
   readFileSync?: (path: string, encoding: 'utf8') => string;
   isSync: boolean;
+  isBrowser: boolean;
 }
 
 export interface StreamOut {
@@ -249,6 +296,7 @@ export interface StreamFS {
 
 export interface StreamService {
   buildOrServe(
+    callName: string,
     serveOptions: types.ServeOptions | null,
     options: types.BuildOptions,
     isTTY: boolean,
@@ -256,6 +304,7 @@ export interface StreamService {
   ): void;
 
   transform(
+    callName: string,
     input: string,
     options: types.TransformOptions,
     isTTY: boolean,
@@ -419,12 +468,18 @@ export function createChannel(streamIn: StreamIn): StreamOut {
     if (streamIn.isSync) throw new Error('Cannot use plugins in synchronous API calls');
 
     let onResolveCallbacks: {
-      [id: number]: (args: types.OnResolveArgs) =>
-        (types.OnResolveResult | null | undefined | Promise<types.OnResolveResult | null | undefined>)
+      [id: number]: {
+        name: string,
+        callback: (args: types.OnResolveArgs) =>
+          (types.OnResolveResult | null | undefined | Promise<types.OnResolveResult | null | undefined>),
+      },
     } = {};
     let onLoadCallbacks: {
-      [id: number]: (args: types.OnLoadArgs) =>
-        (types.OnLoadResult | null | undefined | Promise<types.OnLoadResult | null | undefined>)
+      [id: number]: {
+        name: string,
+        callback: (args: types.OnLoadArgs) =>
+          (types.OnLoadResult | null | undefined | Promise<types.OnLoadResult | null | undefined>),
+      },
     } = {};
     let nextCallbackID = 0;
     let i = 0;
@@ -432,15 +487,20 @@ export function createChannel(streamIn: StreamIn): StreamOut {
     request.plugins = [];
 
     for (let item of plugins) {
-      let name = item.name;
-      let setup = item.setup;
+      let keys: OptionKeys = {};
+      if (typeof item !== 'object') throw new Error(`Plugin at index ${i} must be an object`);
+      let name = getFlag(item, keys, 'name', mustBeString);
+      let setup = getFlag(item, keys, 'setup', mustBeFunction);
+
+      if (typeof name !== 'string' || name === '') throw new Error(`Plugin at index ${i} is missing a name`);
+      if (typeof setup !== 'function') throw new Error(`[${name}] Plugin is missing a setup function`);
+      checkForInvalidFlags(item, keys, `on plugin ${JSON.stringify(name)}`);
+
       let plugin: protocol.BuildPlugin = {
-        name: name + '',
+        name,
         onResolve: [],
         onLoad: [],
       };
-      if (typeof name !== 'string' || name === '') throw new Error(`Plugin at index ${i} is missing a name`);
-      if (typeof setup !== 'function') throw new Error(`[${plugin.name}] Missing a setup function`);
       i++;
 
       setup({
@@ -448,10 +508,10 @@ export function createChannel(streamIn: StreamIn): StreamOut {
           let keys: OptionKeys = {};
           let filter = getFlag(options, keys, 'filter', mustBeRegExp);
           let namespace = getFlag(options, keys, 'namespace', mustBeString);
-          checkForInvalidFlags(options, keys);
-          if (filter == null) throw new Error(`[${plugin.name}] "onResolve" is missing a filter`);
+          checkForInvalidFlags(options, keys, `in onResolve() call for plugin ${JSON.stringify(name)}`);
+          if (filter == null) throw new Error(`[${plugin.name}] onResolve() call is missing a filter`);
           let id = nextCallbackID++;
-          onResolveCallbacks[id] = callback;
+          onResolveCallbacks[id] = { name: name!, callback };
           plugin.onResolve.push({ id, filter: filter.source, namespace: namespace || '' });
         },
 
@@ -459,10 +519,10 @@ export function createChannel(streamIn: StreamIn): StreamOut {
           let keys: OptionKeys = {};
           let filter = getFlag(options, keys, 'filter', mustBeRegExp);
           let namespace = getFlag(options, keys, 'namespace', mustBeString);
-          checkForInvalidFlags(options, keys);
-          if (filter == null) throw new Error(`[${plugin.name}] "onLoad" is missing a filter`);
+          checkForInvalidFlags(options, keys, `in onLoad() call for plugin ${JSON.stringify(name)}`);
+          if (filter == null) throw new Error(`[${plugin.name}] onLoad() call is missing a filter`);
           let id = nextCallbackID++;
-          onLoadCallbacks[id] = callback;
+          onLoadCallbacks[id] = { name: name!, callback };
           plugin.onLoad.push({ id, filter: filter.source, namespace: namespace || '' });
         },
       });
@@ -476,7 +536,7 @@ export function createChannel(streamIn: StreamIn): StreamOut {
           let response: protocol.OnResolveResponse = {};
           for (let id of request.ids) {
             try {
-              let callback = onResolveCallbacks[id];
+              let { name, callback } = onResolveCallbacks[id];
               let result = await callback({
                 path: request.path,
                 importer: request.importer,
@@ -485,7 +545,7 @@ export function createChannel(streamIn: StreamIn): StreamOut {
               });
 
               if (result != null) {
-                if (typeof result !== 'object') throw new Error('Expected resolver plugin to return an object');
+                if (typeof result !== 'object') throw new Error(`Expected onResolve() callback in plugin ${JSON.stringify(name)} to return an object`);
                 let keys: OptionKeys = {};
                 let pluginName = getFlag(result, keys, 'pluginName', mustBeString);
                 let path = getFlag(result, keys, 'path', mustBeString);
@@ -493,15 +553,15 @@ export function createChannel(streamIn: StreamIn): StreamOut {
                 let external = getFlag(result, keys, 'external', mustBeBoolean);
                 let errors = getFlag(result, keys, 'errors', mustBeArray);
                 let warnings = getFlag(result, keys, 'warnings', mustBeArray);
-                checkForInvalidFlags(result, keys);
+                checkForInvalidFlags(result, keys, `from onResolve() callback in plugin ${JSON.stringify(name)}`);
 
                 response.id = id;
                 if (pluginName != null) response.pluginName = pluginName;
                 if (path != null) response.path = path;
                 if (namespace != null) response.namespace = namespace;
                 if (external != null) response.external = external;
-                if (errors != null) response.errors = sanitizeMessages(errors);
-                if (warnings != null) response.warnings = sanitizeMessages(warnings);
+                if (errors != null) response.errors = sanitizeMessages(errors, 'errors');
+                if (warnings != null) response.warnings = sanitizeMessages(warnings, 'warnings');
                 break;
               }
             } catch (e) {
@@ -515,14 +575,14 @@ export function createChannel(streamIn: StreamIn): StreamOut {
           let response: protocol.OnLoadResponse = {};
           for (let id of request.ids) {
             try {
-              let callback = onLoadCallbacks[id];
+              let { name, callback } = onLoadCallbacks[id];
               let result = await callback({
                 path: request.path,
                 namespace: request.namespace,
               });
 
               if (result != null) {
-                if (typeof result !== 'object') throw new Error('Expected loader plugin to return an object');
+                if (typeof result !== 'object') throw new Error(`Expected onLoad() callback in plugin ${JSON.stringify(name)} to return an object`);
                 let keys: OptionKeys = {};
                 let pluginName = getFlag(result, keys, 'pluginName', mustBeString);
                 let contents = getFlag(result, keys, 'contents', mustBeStringOrUint8Array);
@@ -530,7 +590,7 @@ export function createChannel(streamIn: StreamIn): StreamOut {
                 let loader = getFlag(result, keys, 'loader', mustBeString);
                 let errors = getFlag(result, keys, 'errors', mustBeArray);
                 let warnings = getFlag(result, keys, 'warnings', mustBeArray);
-                checkForInvalidFlags(result, keys);
+                checkForInvalidFlags(result, keys, `from onLoad() callback in plugin ${JSON.stringify(name)}`);
 
                 response.id = id;
                 if (pluginName != null) response.pluginName = pluginName;
@@ -538,8 +598,8 @@ export function createChannel(streamIn: StreamIn): StreamOut {
                 else if (contents != null) response.contents = protocol.encodeUTF8(contents);
                 if (resolveDir != null) response.resolveDir = resolveDir;
                 if (loader != null) response.loader = loader;
-                if (errors != null) response.errors = sanitizeMessages(errors);
-                if (warnings != null) response.warnings = sanitizeMessages(warnings);
+                if (errors != null) response.errors = sanitizeMessages(errors, 'errors');
+                if (warnings != null) response.warnings = sanitizeMessages(warnings, 'warnings');
                 break;
               }
             } catch (e) {
@@ -565,6 +625,7 @@ export function createChannel(streamIn: StreamIn): StreamOut {
   let buildServeData = (options: types.ServeOptions, request: protocol.BuildRequest): ServeData => {
     let keys: OptionKeys = {};
     let port = getFlag(options, keys, 'port', mustBeInteger);
+    let host = getFlag(options, keys, 'host', mustBeString);
     let onRequest = getFlag(options, keys, 'onRequest', mustBeFunction);
     let serveID = nextServeID++;
     let onWait: ServeCallbacks['onWait'];
@@ -576,8 +637,9 @@ export function createChannel(streamIn: StreamIn): StreamOut {
       };
     });
     request.serve = { serveID };
-    checkForInvalidFlags(options, keys);
+    checkForInvalidFlags(options, keys, `in serve() call`);
     if (port !== void 0) request.serve.port = port;
+    if (host !== void 0) request.serve.host = host;
     serveCallbacks.set(serveID, {
       onRequest,
       onWait: onWait!,
@@ -597,12 +659,13 @@ export function createChannel(streamIn: StreamIn): StreamOut {
     afterClose,
 
     service: {
-      buildOrServe(serveOptions, options, isTTY, callback) {
+      buildOrServe(callName, serveOptions, options, isTTY, callback) {
         const logLevelDefault = 'info';
         try {
           let key = nextBuildKey++;
-          let [flags, write, plugins, stdin, resolveDir, incremental] = flagsForBuildOptions(options, isTTY, logLevelDefault);
-          let request: protocol.BuildRequest = { command: 'build', key, flags, write, stdin, resolveDir, incremental };
+          let writeDefault = !streamIn.isBrowser;
+          let { flags, write, plugins, stdinContents, stdinResolveDir, incremental } = flagsForBuildOptions(callName, options, isTTY, logLevelDefault, writeDefault);
+          let request: protocol.BuildRequest = { command: 'build', key, flags, write, stdinContents, stdinResolveDir, incremental };
           let serve = serveOptions && buildServeData(serveOptions, request);
           let pluginCleanup = plugins && plugins.length > 0 && handlePlugins(plugins, request, key);
 
@@ -646,6 +709,7 @@ export function createChannel(streamIn: StreamIn): StreamOut {
             return callback(null, result);
           };
 
+          if (write && streamIn.isBrowser) throw new Error(`Cannot enable "write" in the browser`);
           if (incremental && streamIn.isSync) throw new Error(`Cannot use "incremental" with a synchronous build`);
           sendRequest<protocol.BuildRequest, protocol.BuildResponse>(request, (error, response) => {
             if (error) return callback(new Error(error), null);
@@ -653,6 +717,7 @@ export function createChannel(streamIn: StreamIn): StreamOut {
               let serveResponse = response as any as protocol.ServeResponse;
               let result: types.ServeResult = {
                 port: serveResponse.port,
+                host: serveResponse.host,
                 wait: serve.wait,
                 stop: serve.stop,
               };
@@ -670,7 +735,7 @@ export function createChannel(streamIn: StreamIn): StreamOut {
         }
       },
 
-      transform(input, options, isTTY, fs, callback) {
+      transform(callName, input, options, isTTY, fs, callback) {
         const logLevelDefault = 'silent';
 
         // Ideally the "transform()" API would be faster than calling "build()"
@@ -690,12 +755,12 @@ export function createChannel(streamIn: StreamIn): StreamOut {
         // that doesn't work.
         let start = (inputPath: string | null) => {
           try {
-            let flags = flagsForTransformOptions(options, isTTY, logLevelDefault);
+            let flags = flagsForTransformOptions(callName, options, isTTY, logLevelDefault);
             let request: protocol.TransformRequest = {
               command: 'transform',
               flags,
               inputFS: inputPath !== null,
-              input: inputPath !== null ? inputPath : input + '',
+              input: inputPath !== null ? inputPath : input,
             };
             sendRequest<protocol.TransformRequest, protocol.TransformResponse>(request, (error, response) => {
               if (error) return callback(new Error(error), null);
@@ -741,7 +806,7 @@ export function createChannel(streamIn: StreamIn): StreamOut {
             });
           }
         };
-        if (typeof input === 'string' && input.length > 1024 * 1024) {
+        if (input.length > 1024 * 1024) {
           let next = start;
           start = () => fs.writeFile(input, next);
         }
@@ -822,14 +887,15 @@ function failureErrorWithLog(text: string, errors: types.Message[], warnings: ty
   return error;
 }
 
-function sanitizeMessages(messages: types.PartialMessage[]): types.Message[] {
+function sanitizeMessages(messages: types.PartialMessage[], property: string): types.Message[] {
   let messagesClone: types.Message[] = [];
+  let index = 0;
 
   for (const message of messages) {
     let keys: OptionKeys = {};
     let text = getFlag(message, keys, 'text', mustBeString);
     let location = getFlag(message, keys, 'location', mustBeObjectOrNull);
-    checkForInvalidFlags(message, keys);
+    checkForInvalidFlags(message, keys, `in element ${index} of "${property}"`);
 
     let locationClone: types.Message['location'] = null;
     if (location != null) {
@@ -840,7 +906,7 @@ function sanitizeMessages(messages: types.PartialMessage[]): types.Message[] {
       let column = getFlag(location, keys, 'column', mustBeInteger);
       let length = getFlag(location, keys, 'length', mustBeInteger);
       let lineText = getFlag(location, keys, 'lineText', mustBeString);
-      checkForInvalidFlags(location, keys);
+      checkForInvalidFlags(location, keys, `in element ${index} of "${property}"`);
 
       locationClone = {
         file: file || '',
@@ -856,6 +922,7 @@ function sanitizeMessages(messages: types.PartialMessage[]): types.Message[] {
       text: text || '',
       location: locationClone,
     });
+    index++;
   }
 
   return messagesClone;
@@ -872,3 +939,78 @@ function convertOutputFiles({ path, contents }: protocol.BuildOutputFile): types
     },
   }
 }
+
+export function referenceCountedService(getwd: () => string, startService: typeof types.startService): typeof types.startService {
+  interface Entry {
+    promise: Promise<types.Service>;
+    refCount: number;
+  }
+
+  let entries = new Map<string, Entry>();
+
+  return async (options) => {
+    // Mix the current working directory into the key. Some users rely on
+    // calling "process.chdir()" before calling "startService()" to set the
+    // current working directory for that service.
+    let cwd = getwd();
+    let optionsJSON = JSON.stringify(options || {});
+    let key = `${optionsJSON} ${cwd}`;
+    let entry = entries.get(key);
+    let didStop = false;
+
+    let checkWasStopped = () => {
+      if (didStop) {
+        throw new Error('The service was stopped');
+      }
+    };
+
+    // Returns true if this was the last reference
+    let isLastStop = () => {
+      if (!didStop) {
+        didStop = true;
+        if (--entry!.refCount === 0) {
+          entries.delete(key);
+          return true;
+        }
+      }
+      return false;
+    };
+
+    if (entry === void 0) {
+      // Store the promise used to create the service so that multiple
+      // concurrent calls to "startService()" will share the same promise.
+      entry = { promise: startService(JSON.parse(optionsJSON)), refCount: 0 };
+      entries.set(key, entry);
+    }
+
+    ++entry.refCount;
+
+    try {
+      let service = await entry.promise;
+      return {
+        build: (options: any): any => {
+          checkWasStopped();
+          return service.build(options);
+        },
+        serve(serveOptions, buildOptions) {
+          checkWasStopped();
+          return service.serve(serveOptions, buildOptions);
+        },
+        transform(input, options) {
+          checkWasStopped();
+          return service.transform(input, options);
+        },
+        stop() {
+          if (isLastStop()) {
+            service.stop();
+          }
+        },
+      };
+    }
+
+    catch (e) {
+      isLastStop();
+      throw e;
+    }
+  };
+}
