@@ -15,6 +15,13 @@ declare const ESBUILD_VERSION: string;
 declare const WASM: boolean;
 
 let esbuildCommandAndArgs = (): [string, string[]] => {
+  // This feature was added to give external code a way to modify the binary
+  // path without modifying the code itself. Do not remove this because
+  // external code relies on this.
+  if (process.env.ESBUILD_BINARY_PATH) {
+    return [path.resolve(process.env.ESBUILD_BINARY_PATH), []];
+  }
+
   if (WASM) {
     return ['node', [path.join(__dirname, '..', 'bin', 'esbuild')]];
   }
@@ -38,9 +45,9 @@ let esbuildCommandAndArgs = (): [string, string[]] => {
   // to if the original location is replaced by our Yarn 2 compatibility hack.
   // If it exists, we can infer that we are running within Yarn 2 and the
   // JavaScript API should invoke the binary here instead to avoid a slowdown.
-  // This is a performance improvement of about 0.1 seconds for Yarn 2 on my
-  // machine.
-  let pathForYarn2 = path.join(__dirname, 'esbuild');
+  // Calling the binary directly can be over 6x faster than calling the wrapper
+  // script instead.
+  let pathForYarn2 = path.join(__dirname, '..', 'esbuild');
   if (fs.existsSync(pathForYarn2)) {
     return [pathForYarn2, []];
   }
@@ -53,8 +60,8 @@ let isTTY = () => tty.isatty(2);
 
 export let version = ESBUILD_VERSION;
 
-export let build: typeof types.build = options => {
-  return startService().then(service => {
+export let build: typeof types.build = (options: types.BuildOptions): Promise<any> => {
+  return startService().then<types.BuildResult>(service => {
     return service.build(options).then(result => {
       if (result.rebuild) {
         let old = result.rebuild.dispose;
@@ -85,6 +92,7 @@ export let serve: typeof types.serve = (serveOptions, buildOptions) => {
 };
 
 export let transform: typeof types.transform = (input, options) => {
+  input += '';
   return startService().then(service => {
     let promise = service.transform(input, options);
     promise.then(service.stop, service.stop);
@@ -92,9 +100,9 @@ export let transform: typeof types.transform = (input, options) => {
   });
 };
 
-export let buildSync: typeof types.buildSync = options => {
+export let buildSync: typeof types.buildSync = (options: types.BuildOptions): any => {
   let result: types.BuildResult;
-  runServiceSync(service => service.buildOrServe(null, options, isTTY(), (err, res) => {
+  runServiceSync(service => service.buildOrServe('buildSync', null, options, isTTY(), (err, res) => {
     if (err) throw err;
     result = res as types.BuildResult;
   }));
@@ -102,8 +110,9 @@ export let buildSync: typeof types.buildSync = options => {
 };
 
 export let transformSync: typeof types.transformSync = (input, options) => {
+  input += '';
   let result: types.TransformResult;
-  runServiceSync(service => service.transform(input, options || {}, isTTY(), {
+  runServiceSync(service => service.transform('transformSync', input, options || {}, isTTY(), {
     readFile(tempFile, callback) {
       try {
         let contents = fs.readFileSync(tempFile, 'utf8');
@@ -132,11 +141,10 @@ export let transformSync: typeof types.transformSync = (input, options) => {
   return result!;
 };
 
-export let startService: typeof types.startService = options => {
-  if (options) {
-    if (options.wasmURL) throw new Error(`The "wasmURL" option only works in the browser`)
-    if (options.worker) throw new Error(`The "worker" option only works in the browser`)
-  }
+export let startService: typeof types.startService = common.referenceCountedService(() => process.cwd(), options => {
+  options = common.validateServiceOptions(options || {});
+  if (options.wasmURL) throw new Error(`The "wasmURL" option only works in the browser`)
+  if (options.worker) throw new Error(`The "worker" option only works in the browser`)
   let [command, args] = esbuildCommandAndArgs();
   let child = child_process.spawn(command, args.concat(`--service=${ESBUILD_VERSION}`), {
     cwd: process.cwd(),
@@ -149,26 +157,28 @@ export let startService: typeof types.startService = options => {
     },
     readFileSync: fs.readFileSync,
     isSync: false,
+    isBrowser: false,
   });
   child.stdout.on('data', readFromStdout);
   child.stdout.on('end', afterClose);
 
   // Create an asynchronous Promise-based API
   return Promise.resolve({
-    build: options =>
-      new Promise((resolve, reject) =>
-        service.buildOrServe(null, options, isTTY(), (err, res) =>
+    build: (options: types.BuildOptions): Promise<any> =>
+      new Promise<types.BuildResult>((resolve, reject) =>
+        service.buildOrServe('build', null, options, isTTY(), (err, res) =>
           err ? reject(err) : resolve(res as types.BuildResult))),
     serve: (serveOptions, buildOptions) => {
       if (serveOptions === null || typeof serveOptions !== 'object')
         throw new Error('The first argument must be an object')
       return new Promise((resolve, reject) =>
-        service.buildOrServe(serveOptions, buildOptions, isTTY(), (err, res) =>
+        service.buildOrServe('serve', serveOptions, buildOptions, isTTY(), (err, res) =>
           err ? reject(err) : resolve(res as types.ServeResult)))
     },
-    transform: (input, options) =>
-      new Promise((resolve, reject) =>
-        service.transform(input, options || {}, isTTY(), {
+    transform: (input, options) => {
+      input += '';
+      return new Promise((resolve, reject) =>
+        service.transform('transform', input, options || {}, isTTY(), {
           readFile(tempFile, callback) {
             try {
               fs.readFile(tempFile, 'utf8', (err, contents) => {
@@ -191,10 +201,11 @@ export let startService: typeof types.startService = options => {
               callback(null);
             }
           },
-        }, (err, res) => err ? reject(err) : resolve(res!))),
+        }, (err, res) => err ? reject(err) : resolve(res!)));
+    },
     stop() { child.kill(); },
   });
-};
+});
 
 let runServiceSync = (callback: (service: common.StreamService) => void): void => {
   let [command, args] = esbuildCommandAndArgs();
@@ -205,6 +216,7 @@ let runServiceSync = (callback: (service: common.StreamService) => void): void =
       stdin = bytes;
     },
     isSync: true,
+    isBrowser: false,
   });
   callback(service);
   let stdout = child_process.execFileSync(command, args.concat(`--service=${ESBUILD_VERSION}`), {
