@@ -11,8 +11,8 @@ export interface BuildRequest {
   key: number;
   flags: string[];
   write: boolean;
-  stdin: string | null;
-  resolveDir: string | null;
+  stdinContents: string | null;
+  stdinResolveDir: string | null;
   incremental: boolean;
   plugins?: BuildPlugin[];
   serve?: ServeRequest;
@@ -21,10 +21,12 @@ export interface BuildRequest {
 export interface ServeRequest {
   serveID: number;
   port?: number;
+  host?: string;
 }
 
 export interface ServeResponse {
   port: number;
+  host: string;
 }
 
 export interface ServeStopRequest {
