@@ -86,6 +86,13 @@ const (
 	SourceMapExternal
 )
 
+type SourcesContent uint8
+
+const (
+	SourcesContentInclude SourcesContent = iota
+	SourcesContentExclude
+)
+
 type Target uint8
 
 const (
@@ -203,9 +210,11 @@ type BuildOptions struct {
 	ErrorLimit int
 	LogLevel   LogLevel
 
-	Sourcemap SourceMap
-	Target    Target
-	Engines   []Engine
+	Sourcemap      SourceMap
+	SourcesContent SourcesContent
+
+	Target  Target
+	Engines []Engine
 
 	MinifyWhitespace  bool
 	MinifyIdentifiers bool
@@ -281,7 +290,9 @@ type TransformOptions struct {
 	ErrorLimit int
 	LogLevel   LogLevel
 
-	Sourcemap  SourceMap
+	Sourcemap      SourceMap
+	SourcesContent SourcesContent
+
 	Target     Target
 	Format     Format
 	GlobalName string
@@ -325,6 +336,7 @@ func Transform(input string, options TransformOptions) TransformResult {
 
 type ServeOptions struct {
 	Port      uint16
+	Host      string
 	OnRequest func(ServeOnRequestArgs)
 }
 
@@ -338,6 +350,7 @@ type ServeOnRequestArgs struct {
 
 type ServeResult struct {
 	Port uint16
+	Host string
 	Wait func() error
 	Stop func()
 }
