@@ -70,7 +70,7 @@ func validateSourceMap(value SourceMap) config.SourceMap {
 	}
 }
 
-func validateColor(value StderrColor) logger.StderrColor {
+func validateColor(value StderrColor) logger.UseColor {
 	switch value {
 	case ColorIfTerminal:
 		return logger.ColorIfTerminal
@@ -316,12 +316,14 @@ func validateJSX(log logger.Log, text string, name string) []string {
 	return parts
 }
 
-func validateDefines(log logger.Log, defines map[string]string, pureFns []string) *config.ProcessedDefines {
+func validateDefines(log logger.Log, defines map[string]string, pureFns []string) (*config.ProcessedDefines, []config.InjectedDefine) {
 	if len(defines) == 0 && len(pureFns) == 0 {
-		return nil
+		return nil, nil
 	}
 
 	rawDefines := make(map[string]config.DefineData)
+	valueToInject := make(map[string]config.InjectedDefine)
+	var definesToInject []string
 
 	for key, value := range defines {
 		// The key must be a dot-separated identifier list
@@ -337,10 +339,16 @@ func validateDefines(log logger.Log, defines map[string]string, pureFns []string
 			if _, ok := js_lexer.Keywords[value]; !ok {
 				name := value // The closure must close over a variable inside the loop
 				rawDefines[key] = config.DefineData{
-					DefineFunc: func(loc logger.Loc, findSymbol config.FindSymbol) js_ast.E {
-						return &js_ast.EIdentifier{Ref: findSymbol(loc, name)}
+					DefineFunc: func(args config.DefineArgs) js_ast.E {
+						return &js_ast.EIdentifier{Ref: args.FindSymbol(args.Loc, name)}
 					},
 				}
+
+				// Try to be helpful for common mistakes
+				if key == "process.env.NODE_ENV" {
+					log.AddWarning(nil, logger.Loc{}, fmt.Sprintf(
+						"%q is defined as an identifier instead of a string (surround %q with double quotes to get a string)", key, value))
+				}
 				continue
 			}
 		}
@@ -349,29 +357,44 @@ func validateDefines(log logger.Log, defines map[string]string, pureFns []string
 		source := logger.Source{Contents: value}
 		expr, ok := js_parser.ParseJSON(logger.NewDeferLog(), source, js_parser.JSONOptions{})
 		if !ok {
-			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid define value: %q", value))
+			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid define value (must be valid JSON syntax or a single identifier): %s", value))
 			continue
 		}
 
-		// Only allow atoms for now
 		var fn config.DefineFunc
 		switch e := expr.Data.(type) {
+		// These values are inserted inline, and can participate in constant folding
 		case *js_ast.ENull:
-			fn = func(logger.Loc, config.FindSymbol) js_ast.E { return &js_ast.ENull{} }
+			fn = func(config.DefineArgs) js_ast.E { return &js_ast.ENull{} }
 		case *js_ast.EBoolean:
-			fn = func(logger.Loc, config.FindSymbol) js_ast.E { return &js_ast.EBoolean{Value: e.Value} }
+			fn = func(config.DefineArgs) js_ast.E { return &js_ast.EBoolean{Value: e.Value} }
 		case *js_ast.EString:
-			fn = func(logger.Loc, config.FindSymbol) js_ast.E { return &js_ast.EString{Value: e.Value} }
+			fn = func(config.DefineArgs) js_ast.E { return &js_ast.EString{Value: e.Value} }
 		case *js_ast.ENumber:
-			fn = func(logger.Loc, config.FindSymbol) js_ast.E { return &js_ast.ENumber{Value: e.Value} }
-		default:
-			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid define value: %q", value))
+			fn = func(config.DefineArgs) js_ast.E { return &js_ast.ENumber{Value: e.Value} }
+
+		// These values are extracted into a shared symbol reference
+		case *js_ast.EArray, *js_ast.EObject:
+			definesToInject = append(definesToInject, key)
+			valueToInject[key] = config.InjectedDefine{Source: source, Data: e, Name: key}
 			continue
 		}
 
 		rawDefines[key] = config.DefineData{DefineFunc: fn}
 	}
 
+	// Sort injected defines for determinism, since the imports will be injected
+	// into every file in the order that we return them from this function
+	injectedDefines := make([]config.InjectedDefine, len(definesToInject))
+	sort.Strings(definesToInject)
+	for i, key := range definesToInject {
+		index := i // Capture this for the closure below
+		injectedDefines[i] = valueToInject[key]
+		rawDefines[key] = config.DefineData{DefineFunc: func(args config.DefineArgs) js_ast.E {
+			return &js_ast.EIdentifier{Ref: args.SymbolForDefine(index)}
+		}}
+	}
+
 	for _, key := range pureFns {
 		// The key must be a dot-separated identifier list
 		for _, part := range strings.Split(key, ".") {
@@ -390,7 +413,7 @@ func validateDefines(log logger.Log, defines map[string]string, pureFns []string
 	// Processing defines is expensive. Process them once here so the same object
 	// can be shared between all parsers we create using these arguments.
 	processed := config.ProcessDefines(rawDefines)
-	return &processed
+	return &processed, injectedDefines
 }
 
 func validatePath(log logger.Log, fs fs.FS, relPath string) string {
@@ -483,7 +506,7 @@ func convertMessagesToInternal(msgs []logger.Msg, kind logger.MsgKind, messages
 // Build API
 
 func buildImpl(buildOpts BuildOptions) BuildResult {
-	logOptions := logger.StderrOptions{
+	logOptions := logger.OutputOptions{
 		IncludeSource: true,
 		ErrorLimit:    buildOpts.ErrorLimit,
 		Color:         validateColor(buildOpts.Color),
@@ -501,13 +524,14 @@ func rebuildImpl(
 	buildOpts BuildOptions,
 	caches *cache.CacheSet,
 	plugins []config.Plugin,
-	logOptions logger.StderrOptions,
+	logOptions logger.OutputOptions,
 	log logger.Log,
 ) BuildResult {
 	// Convert and validate the buildOpts
 	realFS := fs.RealFS()
 	jsFeatures, cssFeatures := validateFeatures(log, buildOpts.Target, buildOpts.Engines)
 	outJS, outCSS := validateOutputExtensions(log, buildOpts.OutExtensions)
+	defines, injectedDefines := validateDefines(log, buildOpts.Define, buildOpts.Pure)
 	options := config.Options{
 		UnsupportedJSFeatures:  jsFeatures,
 		UnsupportedCSSFeatures: cssFeatures,
@@ -515,34 +539,36 @@ func rebuildImpl(
 			Factory:  validateJSX(log, buildOpts.JSXFactory, "factory"),
 			Fragment: validateJSX(log, buildOpts.JSXFragment, "fragment"),
 		},
-		Defines:              validateDefines(log, buildOpts.Define, buildOpts.Pure),
-		Platform:             validatePlatform(buildOpts.Platform),
-		SourceMap:            validateSourceMap(buildOpts.Sourcemap),
-		MangleSyntax:         buildOpts.MinifySyntax,
-		RemoveWhitespace:     buildOpts.MinifyWhitespace,
-		MinifyIdentifiers:    buildOpts.MinifyIdentifiers,
-		ASCIIOnly:            validateASCIIOnly(buildOpts.Charset),
-		IgnoreDCEAnnotations: validateIgnoreDCEAnnotations(buildOpts.TreeShaking),
-		GlobalName:           validateGlobalName(log, buildOpts.GlobalName),
-		CodeSplitting:        buildOpts.Splitting,
-		OutputFormat:         validateFormat(buildOpts.Format),
-		AbsOutputFile:        validatePath(log, realFS, buildOpts.Outfile),
-		AbsOutputDir:         validatePath(log, realFS, buildOpts.Outdir),
-		AbsOutputBase:        validatePath(log, realFS, buildOpts.Outbase),
-		AbsMetadataFile:      validatePath(log, realFS, buildOpts.Metafile),
-		OutputExtensionJS:    outJS,
-		OutputExtensionCSS:   outCSS,
-		ExtensionToLoader:    validateLoaders(log, buildOpts.Loader),
-		ExtensionOrder:       validateResolveExtensions(log, buildOpts.ResolveExtensions),
-		ExternalModules:      validateExternals(log, realFS, buildOpts.External),
-		TsConfigOverride:     validatePath(log, realFS, buildOpts.Tsconfig),
-		MainFields:           buildOpts.MainFields,
-		PublicPath:           buildOpts.PublicPath,
-		KeepNames:            buildOpts.KeepNames,
-		InjectAbsPaths:       make([]string, len(buildOpts.Inject)),
-		Banner:               buildOpts.Banner,
-		Footer:               buildOpts.Footer,
-		Plugins:              plugins,
+		Defines:               defines,
+		InjectedDefines:       injectedDefines,
+		Platform:              validatePlatform(buildOpts.Platform),
+		SourceMap:             validateSourceMap(buildOpts.Sourcemap),
+		ExcludeSourcesContent: buildOpts.SourcesContent == SourcesContentExclude,
+		MangleSyntax:          buildOpts.MinifySyntax,
+		RemoveWhitespace:      buildOpts.MinifyWhitespace,
+		MinifyIdentifiers:     buildOpts.MinifyIdentifiers,
+		ASCIIOnly:             validateASCIIOnly(buildOpts.Charset),
+		IgnoreDCEAnnotations:  validateIgnoreDCEAnnotations(buildOpts.TreeShaking),
+		GlobalName:            validateGlobalName(log, buildOpts.GlobalName),
+		CodeSplitting:         buildOpts.Splitting,
+		OutputFormat:          validateFormat(buildOpts.Format),
+		AbsOutputFile:         validatePath(log, realFS, buildOpts.Outfile),
+		AbsOutputDir:          validatePath(log, realFS, buildOpts.Outdir),
+		AbsOutputBase:         validatePath(log, realFS, buildOpts.Outbase),
+		AbsMetadataFile:       validatePath(log, realFS, buildOpts.Metafile),
+		OutputExtensionJS:     outJS,
+		OutputExtensionCSS:    outCSS,
+		ExtensionToLoader:     validateLoaders(log, buildOpts.Loader),
+		ExtensionOrder:        validateResolveExtensions(log, buildOpts.ResolveExtensions),
+		ExternalModules:       validateExternals(log, realFS, buildOpts.External),
+		TsConfigOverride:      validatePath(log, realFS, buildOpts.Tsconfig),
+		MainFields:            buildOpts.MainFields,
+		PublicPath:            buildOpts.PublicPath,
+		KeepNames:             buildOpts.KeepNames,
+		InjectAbsPaths:        make([]string, len(buildOpts.Inject)),
+		Banner:                buildOpts.Banner,
+		Footer:                buildOpts.Footer,
+		Plugins:               plugins,
 	}
 	for i, path := range buildOpts.Inject {
 		options.InjectAbsPaths[i] = validatePath(log, realFS, path)
@@ -550,13 +576,10 @@ func rebuildImpl(
 	if options.PublicPath != "" && !strings.HasSuffix(options.PublicPath, "/") && !strings.HasSuffix(options.PublicPath, "\\") {
 		options.PublicPath += "/"
 	}
-	entryPaths := make([]string, len(buildOpts.EntryPoints))
-	for i, entryPoint := range buildOpts.EntryPoints {
-		entryPaths[i] = validatePath(log, realFS, entryPoint)
-	}
-	entryPathCount := len(buildOpts.EntryPoints)
+	entryPoints := append([]string{}, buildOpts.EntryPoints...)
+	entryPointCount := len(entryPoints)
 	if buildOpts.Stdin != nil {
-		entryPathCount++
+		entryPointCount++
 		options.Stdin = &config.StdinInfo{
 			Loader:        validateLoader(buildOpts.Stdin.Loader),
 			Contents:      buildOpts.Stdin.Contents,
@@ -565,7 +588,7 @@ func rebuildImpl(
 		}
 	}
 
-	if options.AbsOutputDir == "" && entryPathCount > 1 {
+	if options.AbsOutputDir == "" && entryPointCount > 1 {
 		log.AddError(nil, logger.Loc{},
 			"Must use \"outdir\" when there are multiple input files")
 	} else if options.AbsOutputDir == "" && options.CodeSplitting {
@@ -631,7 +654,7 @@ func rebuildImpl(
 	if !log.HasErrors() {
 		// Scan over the bundle
 		resolver := resolver.NewResolver(realFS, log, caches, options)
-		bundle := bundler.ScanBundle(log, realFS, resolver, caches, entryPaths, options)
+		bundle := bundler.ScanBundle(log, realFS, resolver, caches, entryPoints, options)
 
 		// Stop now if there were errors
 		if !log.HasErrors() {
@@ -713,7 +736,7 @@ func rebuildImpl(
 // Transform API
 
 func transformImpl(input string, transformOpts TransformOptions) TransformResult {
-	log := logger.NewStderrLog(logger.StderrOptions{
+	log := logger.NewStderrLog(logger.OutputOptions{
 		IncludeSource: true,
 		ErrorLimit:    transformOpts.ErrorLimit,
 		Color:         validateColor(transformOpts.Color),
@@ -762,12 +785,15 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 
 	// Convert and validate the transformOpts
 	jsFeatures, cssFeatures := validateFeatures(log, transformOpts.Target, transformOpts.Engines)
+	defines, injectedDefines := validateDefines(log, transformOpts.Define, transformOpts.Pure)
 	options := config.Options{
 		UnsupportedJSFeatures:   jsFeatures,
 		UnsupportedCSSFeatures:  cssFeatures,
 		JSX:                     jsx,
-		Defines:                 validateDefines(log, transformOpts.Define, transformOpts.Pure),
+		Defines:                 defines,
+		InjectedDefines:         injectedDefines,
 		SourceMap:               validateSourceMap(transformOpts.Sourcemap),
+		ExcludeSourcesContent:   transformOpts.SourcesContent == SourcesContentExclude,
 		OutputFormat:            validateFormat(transformOpts.Format),
 		GlobalName:              validateGlobalName(log, transformOpts.GlobalName),
 		MangleSyntax:            transformOpts.MinifySyntax,
@@ -1026,7 +1052,7 @@ func (h *apiHandler) notifyRequest(duration time.Duration, req *http.Request, st
 }
 
 func errorsToString(errors []Message) string {
-	stderrOptions := logger.StderrOptions{IncludeSource: true}
+	stderrOptions := logger.OutputOptions{IncludeSource: true}
 	terminalOptions := logger.TerminalInfo{}
 	sb := strings.Builder{}
 	limit := 5
@@ -1145,6 +1171,9 @@ func serveImpl(serveOptions ServeOptions, buildOptions BuildOptions) (ServeResul
 	// Pick the port
 	var listener net.Listener
 	host := "127.0.0.1"
+	if serveOptions.Host != "" {
+		host = serveOptions.Host
+	}
 	if serveOptions.Port == 0 {
 		// Default to picking a "800X" port
 		for port := 8000; port <= 8009; port++ {
@@ -1168,9 +1197,10 @@ func serveImpl(serveOptions ServeOptions, buildOptions BuildOptions) (ServeResul
 
 	// Extract the real port in case we passed a port of "0"
 	var result ServeResult
-	if _, text, err := net.SplitHostPort(addr); err == nil {
+	if host, text, err := net.SplitHostPort(addr); err == nil {
 		if port, err := strconv.ParseInt(text, 10, 32); err == nil {
 			result.Port = uint16(port)
+			result.Host = host
 		}
 	}
 
