@@ -3,12 +3,15 @@ package cli
 import (
 	"fmt"
 	"io/ioutil"
+	"net"
 	"os"
+	"path/filepath"
 	"sort"
 	"strconv"
 	"strings"
+	"time"
 
-	"github.com/evanw/esbuild/internal/helpers"
+	"github.com/evanw/esbuild/internal/cli_helpers"
 	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/pkg/api"
 )
@@ -140,6 +143,23 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 			}
 			hasBareSourceMapFlag = false
 
+		case strings.HasPrefix(arg, "--sources-content="):
+			value := arg[len("--sources-content="):]
+			var sourcesContent api.SourcesContent
+			switch value {
+			case "false":
+				sourcesContent = api.SourcesContentExclude
+			case "true":
+				sourcesContent = api.SourcesContentInclude
+			default:
+				return fmt.Errorf("Invalid sources content: %q (valid: false, true)", value)
+			}
+			if buildOpts != nil {
+				buildOpts.SourcesContent = sourcesContent
+			} else {
+				transformOpts.SourcesContent = sourcesContent
+			}
+
 		case strings.HasPrefix(arg, "--sourcefile="):
 			if buildOpts != nil {
 				if buildOpts.Stdin == nil {
@@ -211,7 +231,7 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 				return fmt.Errorf("Missing \"=\": %q", value)
 			}
 			ext, text := value[:equals], value[equals+1:]
-			loader, err := helpers.ParseLoader(text)
+			loader, err := cli_helpers.ParseLoader(text)
 			if err != nil {
 				return err
 			}
@@ -219,7 +239,7 @@ func parseOptionsImpl(osArgs []string, buildOpts *api.BuildOptions, transformOpt
 
 		case strings.HasPrefix(arg, "--loader="):
 			value := arg[len("--loader="):]
-			loader, err := helpers.ParseLoader(value)
+			loader, err := cli_helpers.ParseLoader(value)
 			if err != nil {
 				return err
 			}
@@ -499,8 +519,12 @@ func parseOptionsForRun(osArgs []string) (*api.BuildOptions, *api.TransformOptio
 }
 
 func runImpl(osArgs []string) int {
-	// Special-case running a server
+	shouldPrintSummary := false
+	start := time.Now()
+	end := 0
+
 	for i, arg := range osArgs {
+		// Special-case running a server
 		if arg == "--serve" {
 			arg = "--serve=0"
 		}
@@ -513,7 +537,17 @@ func runImpl(osArgs []string) int {
 			}
 			return 0
 		}
+
+		// Filter out the "--summary" flag
+		if arg == "--summary" {
+			shouldPrintSummary = true
+			continue
+		}
+
+		osArgs[end] = arg
+		end++
 	}
+	osArgs = osArgs[:end]
 
 	buildOptions, transformOptions, err := parseOptionsForRun(osArgs)
 
@@ -549,6 +583,11 @@ func runImpl(osArgs []string) int {
 			return 1
 		}
 
+		// Print a summary to stderr
+		if shouldPrintSummary {
+			printSummary(osArgs, result.OutputFiles, start)
+		}
+
 	case transformOptions != nil:
 		// Read the input from stdin
 		bytes, err := ioutil.ReadAll(os.Stdin)
@@ -567,6 +606,11 @@ func runImpl(osArgs []string) int {
 		// Write the output to stdout
 		os.Stdout.Write(result.Code)
 
+		// Print a summary to stderr
+		if shouldPrintSummary {
+			printSummary(osArgs, nil, start)
+		}
+
 	case err != nil:
 		logger.PrintErrorToStderr(osArgs, err.Error())
 		return 1
@@ -575,7 +619,56 @@ func runImpl(osArgs []string) int {
 	return 0
 }
 
-func serveImpl(portText string, osArgs []string) error {
+func printSummary(osArgs []string, outputFiles []api.OutputFile, start time.Time) {
+	var table logger.SummaryTable = make([]logger.SummaryTableEntry, len(outputFiles))
+
+	if len(outputFiles) > 0 {
+		cwd, _ := os.Getwd()
+
+		for i, file := range outputFiles {
+			path, err := filepath.Rel(cwd, file.Path)
+			if err != nil {
+				path = file.Path
+			}
+			dir, base := filepath.Split(path)
+			n := len(file.Contents)
+			var size string
+			if n < 1024 {
+				size = fmt.Sprintf("%db ", n)
+			} else if n < 1024*1024 {
+				size = fmt.Sprintf("%.1fkb", float64(n)/(1024))
+			} else if n < 1024*1024*1024 {
+				size = fmt.Sprintf("%.1fmb", float64(n)/(1024*1024))
+			} else {
+				size = fmt.Sprintf("%.1fgb", float64(n)/(1024*1024*1024))
+			}
+			table[i] = logger.SummaryTableEntry{
+				Dir:         dir,
+				Base:        base,
+				Size:        size,
+				Bytes:       n,
+				IsSourceMap: strings.HasSuffix(base, ".map"),
+			}
+		}
+	}
+
+	logger.PrintSummary(osArgs, table, start)
+}
+
+func serveImpl(serveText string, osArgs []string) error {
+	host := ""
+	portText := serveText
+
+	// Specifying the host is optional
+	if strings.ContainsRune(serveText, ':') {
+		var err error
+		host, portText, err = net.SplitHostPort(serveText)
+		if err != nil {
+			return err
+		}
+	}
+
+	// Parse the port
 	port, err := strconv.ParseInt(portText, 10, 32)
 	if err != nil {
 		return err
@@ -597,8 +690,9 @@ func serveImpl(portText string, osArgs []string) error {
 
 	serveOptions := api.ServeOptions{
 		Port: uint16(port),
+		Host: host,
 		OnRequest: func(args api.ServeOnRequestArgs) {
-			logger.PrintTextToStderr(logger.LevelInfo, osArgs, func(colors logger.Colors) string {
+			logger.PrintText(os.Stderr, logger.LevelInfo, osArgs, func(colors logger.Colors) string {
 				statusColor := colors.Red
 				if args.Status == 200 {
 					statusColor = colors.Green
@@ -616,9 +710,9 @@ func serveImpl(portText string, osArgs []string) error {
 	}
 
 	// Show what actually got bound if the port was 0
-	logger.PrintTextToStderr(logger.LevelInfo, osArgs, func(colors logger.Colors) string {
-		return fmt.Sprintf("%s\n > %shttp://localhost:%d/%s\n\n",
-			colors.Default, colors.Underline, result.Port, colors.Default)
+	logger.PrintText(os.Stderr, logger.LevelInfo, osArgs, func(colors logger.Colors) string {
+		return fmt.Sprintf("%s\n > %shttp://%s:%d/%s\n\n",
+			colors.Default, colors.Underline, result.Host, result.Port, colors.Default)
 	})
 	return result.Wait()
 }
