@@ -1,8 +1,8 @@
-const { installForTests } = require('./esbuild')
-const rimraf = require('rimraf')
+const { installForTests, removeRecursiveSync } = require('./esbuild')
 const assert = require('assert')
 const path = require('path')
 const util = require('util')
+const url = require('url')
 const fs = require('fs')
 
 const readFileAsync = util.promisify(fs.readFile)
@@ -83,6 +83,166 @@ let pluginTests = {
     }
   },
 
+  async pluginMissingName({ esbuild }) {
+    try {
+      await esbuild.build({
+        entryPoints: [],
+        logLevel: 'silent',
+        plugins: [{
+          setup(build) {
+          },
+        }],
+      })
+    } catch (e) {
+      assert.strictEqual(e.message, 'Plugin at index 0 is missing a name')
+    }
+  },
+
+  async pluginMissingSetup({ esbuild }) {
+    try {
+      await esbuild.build({
+        entryPoints: [],
+        logLevel: 'silent',
+        plugins: [{
+          name: 'x',
+        }],
+      })
+    } catch (e) {
+      assert.strictEqual(e.message, '[x] Plugin is missing a setup function')
+    }
+  },
+
+  async badPluginProperty({ esbuild }) {
+    try {
+      await esbuild.build({
+        entryPoints: [],
+        logLevel: 'silent',
+        plugins: [{
+          name: 'x',
+          someRandomProperty: void 0,
+          setup(build) {
+          },
+        }],
+      })
+    } catch (e) {
+      assert.strictEqual(e.message, `Invalid option on plugin "x": "someRandomProperty"`)
+    }
+  },
+
+  async badPluginOnResolveProperty({ esbuild }) {
+    try {
+      await esbuild.build({
+        entryPoints: ['entry'],
+        logLevel: 'silent',
+        plugins: [{
+          name: 'x',
+          setup(build) {
+            build.onResolve({ whatIsThis: void 0 }, () => {
+            })
+          },
+        }],
+      })
+    } catch (e) {
+      assert.strictEqual(e.message, `Invalid option in onResolve() call for plugin "x": "whatIsThis"`)
+    }
+
+    try {
+      await esbuild.build({
+        entryPoints: ['entry'],
+        logLevel: 'silent',
+        write: false,
+        plugins: [{
+          name: 'x',
+          setup(build) {
+            build.onResolve({ filter: /.*/ }, () => {
+              return '/'
+            })
+          },
+        }],
+      })
+    } catch (e) {
+      assert(e.message.endsWith('error: [x] Expected onResolve() callback in plugin "x" to return an object'), e.message)
+    }
+
+    try {
+      await esbuild.build({
+        entryPoints: ['entry'],
+        logLevel: 'silent',
+        write: false,
+        plugins: [{
+          name: 'x',
+          setup(build) {
+            build.onResolve({ filter: /.*/ }, () => {
+              return { thisIsWrong: void 0 }
+            })
+          },
+        }],
+      })
+    } catch (e) {
+      assert(e.message.endsWith('error: [x] Invalid option from onResolve() callback in plugin "x": "thisIsWrong"'), e.message)
+    }
+  },
+
+  async badPluginOnLoadProperty({ esbuild }) {
+    try {
+      await esbuild.build({
+        entryPoints: ['entry'],
+        logLevel: 'silent',
+        plugins: [{
+          name: 'x',
+          setup(build) {
+            build.onLoad({ whatIsThis: void 0 }, () => {
+            })
+          },
+        }],
+      })
+    } catch (e) {
+      assert.strictEqual(e.message, `Invalid option in onLoad() call for plugin "x": "whatIsThis"`)
+    }
+
+    try {
+      await esbuild.build({
+        entryPoints: ['entry'],
+        logLevel: 'silent',
+        write: false,
+        plugins: [{
+          name: 'x',
+          setup(build) {
+            build.onResolve({ filter: /.*/ }, () => {
+              return { path: 'y', namespace: 'z' }
+            })
+            build.onLoad({ filter: /.*/ }, () => {
+              return ""
+            })
+          },
+        }],
+      })
+    } catch (e) {
+      assert(e.message.endsWith(`error: [x] Expected onLoad() callback in plugin "x" to return an object`), e.message)
+    }
+
+    try {
+      await esbuild.build({
+        entryPoints: ['entry'],
+        logLevel: 'silent',
+        write: false,
+        plugins: [{
+          name: 'x',
+          setup(build) {
+            build.onResolve({ filter: /.*/ }, () => {
+              return { path: 'y', namespace: 'z' }
+            })
+            build.onLoad({ filter: /.*/ }, () => {
+              return { thisIsWrong: void 0 }
+            })
+          },
+        }],
+      })
+    } catch (e) {
+      assert(e.message.endsWith('error: [x] Invalid option from onLoad() callback in plugin "x": "thisIsWrong"'), e.message)
+    }
+  },
+
   async basicLoader({ esbuild, testDir }) {
     const input = path.join(testDir, 'in.js')
     const custom = path.join(testDir, 'example.custom')
@@ -557,6 +717,115 @@ let pluginTests = {
     assert.strictEqual(await result.default(103, 20), 123)
   },
 
+  async virtualEntryPoints({ esbuild, testDir }) {
+    const result = await esbuild.build({
+      entryPoints: ['1', '2', 'a<>:"|?*b', 'a/b/c.d.e'],
+      bundle: true, write: false, outdir: testDir, format: 'esm', plugins: [{
+        name: 'name',
+        setup(build) {
+          build.onResolve({ filter: /.*/ }, args => {
+            return { path: `input ${args.path}`, namespace: 'virtual-ns' }
+          })
+          build.onLoad({ filter: /.*/, namespace: 'virtual-ns' }, args => {
+            return { contents: `console.log(${JSON.stringify(args.path)})` }
+          })
+        },
+      }],
+    })
+    assert.strictEqual(result.outputFiles.length, 4)
+    assert.strictEqual(result.outputFiles[0].path, path.join(testDir, 'input 1.js'))
+    assert.strictEqual(result.outputFiles[1].path, path.join(testDir, 'input 2.js'))
+    assert.strictEqual(result.outputFiles[2].path, path.join(testDir, 'input a_b.js'))
+    assert.strictEqual(result.outputFiles[3].path, path.join(testDir, 'c.d.js'))
+    assert.strictEqual(result.outputFiles[0].text, `// virtual-ns:input 1\nconsole.log("input 1");\n`)
+    assert.strictEqual(result.outputFiles[1].text, `// virtual-ns:input 2\nconsole.log("input 2");\n`)
+    assert.strictEqual(result.outputFiles[2].text, `// virtual-ns:input a<>:"|?*b\nconsole.log('input a<>:"|?*b');\n`)
+    assert.strictEqual(result.outputFiles[3].text, `// virtual-ns:input a/b/c.d.e\nconsole.log("input a/b/c.d.e");\n`)
+  },
+
+  async stdinImporter({ esbuild, testDir }) {
+    const output = path.join(testDir, 'out.js')
+    await esbuild.build({
+      stdin: {
+        contents: `import x from "plugin"; export default x`,
+        sourcefile: 'stdin-sourcefile',
+      },
+      bundle: true, outfile: output, format: 'cjs', plugins: [{
+        name: 'name',
+        setup(build) {
+          build.onResolve({ filter: /^plugin$/ }, args => {
+            assert.strictEqual(args.namespace, '')
+            assert.strictEqual(args.importer, 'stdin-sourcefile')
+            assert.strictEqual(args.resolveDir, '')
+            assert.strictEqual(args.path, 'plugin')
+            return { path: args.path, namespace: 'worked' }
+          })
+          build.onLoad({ filter: /.*/, namespace: 'worked' }, () => {
+            return { contents: `export default 123` }
+          })
+        },
+      }],
+    })
+    const result = require(output)
+    assert.strictEqual(result.default, 123)
+  },
+
+  async stdinImporterResolveDir({ esbuild, testDir }) {
+    const output = path.join(testDir, 'out.js')
+    await esbuild.build({
+      stdin: {
+        contents: `import x from "plugin"; export default x`,
+        sourcefile: 'stdin-sourcefile',
+        resolveDir: testDir,
+      },
+      bundle: true, outfile: output, format: 'cjs', plugins: [{
+        name: 'name',
+        setup(build) {
+          build.onResolve({ filter: /^plugin$/ }, args => {
+            assert.strictEqual(args.namespace, 'file')
+            assert.strictEqual(args.importer, path.join(testDir, 'stdin-sourcefile'))
+            assert.strictEqual(args.resolveDir, testDir)
+            assert.strictEqual(args.path, 'plugin')
+            return { path: args.path, namespace: 'worked' }
+          })
+          build.onLoad({ filter: /.*/, namespace: 'worked' }, () => {
+            return { contents: `export default 123` }
+          })
+        },
+      }],
+    })
+    const result = require(output)
+    assert.strictEqual(result.default, 123)
+  },
+
+  async stdinAbsoluteImporterResolveDir({ esbuild, testDir }) {
+    const output = path.join(testDir, 'out.js')
+    await esbuild.build({
+      stdin: {
+        contents: `import x from "plugin"; export default x`,
+        sourcefile: path.join(testDir, 'stdin-sourcefile'),
+        resolveDir: testDir,
+      },
+      bundle: true, outfile: output, format: 'cjs', plugins: [{
+        name: 'name',
+        setup(build) {
+          build.onResolve({ filter: /^plugin$/ }, args => {
+            assert.strictEqual(args.namespace, 'file')
+            assert.strictEqual(args.importer, path.join(testDir, 'stdin-sourcefile'))
+            assert.strictEqual(args.resolveDir, testDir)
+            assert.strictEqual(args.path, 'plugin')
+            return { path: args.path, namespace: 'worked' }
+          })
+          build.onLoad({ filter: /.*/, namespace: 'worked' }, () => {
+            return { contents: `export default 123` }
+          })
+        },
+      }],
+    })
+    const result = require(output)
+    assert.strictEqual(result.default, 123)
+  },
+
   async stdinRelative({ esbuild, testDir }) {
     const output = path.join(testDir, 'out.js')
     await esbuild.build({
@@ -607,11 +876,106 @@ let pluginTests = {
     const result = require(output)
     assert.strictEqual(result.default, 123)
   },
+
+  async externalRequire({ esbuild, testDir }) {
+    const externalPlugin = external => ({
+      name: 'external',
+      setup(build) {
+        let escape = text => `^${text.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&')}$`
+        let filter = new RegExp(external.map(escape).join('|'))
+        build.onResolve({ filter: /.*/, namespace: 'external' }, args => ({
+          path: args.path, external: true
+        }))
+        build.onResolve({ filter }, args => ({
+          path: args.path, namespace: 'external'
+        }))
+        build.onLoad({ filter: /.*/, namespace: 'external' }, args => ({
+          contents: `import * as all from ${JSON.stringify(args.path)}; module.exports = all`
+        }))
+      },
+    })
+    const outfile = path.join(testDir, 'out', 'output.mjs')
+    await esbuild.build({
+      stdin: {
+        contents: `
+          const fs = require('fs')
+          const url = require('url')
+          const path = require('path')
+          export default fs.readdirSync(path.dirname(url.fileURLToPath(import.meta.url)))
+        `,
+      },
+      bundle: true, outfile, format: 'esm', plugins: [
+        externalPlugin(['fs', 'url', 'path'])
+      ],
+    })
+    const result = await import(url.pathToFileURL(outfile))
+    assert.deepStrictEqual(result.default, [path.basename(outfile)])
+  },
+
+  async newlineInPath({ esbuild }) {
+    // Using a path with a newline shouldn't cause a syntax error when the path is printed in a comment
+    for (let nl of ['\r', '\n', '\r\n', '\u2028', '\u2029']) {
+      let problem = `a b${nl}c d`
+      const plugin = {
+        name: 'test',
+        setup(build) {
+          build.onResolve({ filter: /.*/ }, args => ({
+            path: args.path, namespace: 'test',
+          }))
+          build.onLoad({ filter: /.*/, namespace: 'test' }, args => ({
+            contents: `return ${JSON.stringify(args.path)}`
+          }))
+        },
+      }
+      let result = await esbuild.build({
+        entryPoints: [problem],
+        bundle: true, write: false, format: 'cjs', plugins: [plugin],
+      })
+      let value = new Function(result.outputFiles[0].text)()
+      assert.deepStrictEqual(value, problem)
+    }
+  },
+
+  async newlineInNamespace({ esbuild }) {
+    // Using a namespace with a newline shouldn't cause a syntax error when the namespace is printed in a comment
+    for (let nl of ['\r', '\n', '\r\n', '\u2028', '\u2029']) {
+      let problem = `a b${nl}c d`
+      const plugin = {
+        name: 'test',
+        setup(build) {
+          build.onResolve({ filter: /.*/ }, args => ({
+            path: args.path, namespace: problem,
+          }))
+          build.onLoad({ filter: /.*/, namespace: problem }, args => ({
+            contents: `return ${JSON.stringify(args.namespace)}`
+          }))
+        },
+      }
+      let result = await esbuild.build({
+        entryPoints: ['entry'],
+        bundle: true, write: false, format: 'cjs', plugins: [plugin],
+      })
+      let value = new Function(result.outputFiles[0].text)()
+      assert.deepStrictEqual(value, problem)
+    }
+  },
 }
 
 async function main() {
+  const esbuild = installForTests()
+
+  // Create a fresh test directory
+  removeRecursiveSync(rootTestDir)
+  fs.mkdirSync(rootTestDir)
+
+  // Time out these tests after 5 minutes. This exists to help debug test hangs in CI.
+  let minutes = 5
+  let timeout = setTimeout(() => {
+    console.error(`❌ js api tests timed out after ${minutes} minutes, exiting...`)
+    process.exit(1)
+  }, minutes * 60 * 1000)
+
   // Start the esbuild service
-  const esbuild = installForTests(rootTestDir)
   const service = await esbuild.startService()
 
   // Run all tests concurrently
@@ -620,7 +984,7 @@ async function main() {
     try {
       await mkdirAsync(testDir)
       await fn({ esbuild, service, testDir })
-      rimraf.sync(testDir, { disableGlob: true })
+      removeRecursiveSync(testDir)
       return true
     } catch (e) {
       console.error(`❌ ${name}: ${e && e.message || e}`)
@@ -638,8 +1002,10 @@ async function main() {
     process.exit(1)
   } else {
     console.log(`✅ plugin tests passed`)
-    rimraf.sync(rootTestDir, { disableGlob: true })
+    removeRecursiveSync(rootTestDir)
   }
+
+  clearTimeout(timeout);
 }
 
 main().catch(e => setTimeout(() => { throw e }))
